@@ -1,6 +1,6 @@
 ;;;; batch-driver.scm - Driver procedure for the compiler
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -44,16 +44,17 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   compiler-cleanup-hook check-global-exports disabled-warnings check-global-imports
-  file-io-only undefine-shadowed-macros
+  foreign-type-table-size file-io-only parse-easy-ffi
   unit-name insert-timer-checks used-units inline-max-size
-  debugging perform-lambda-lifting! disable-stack-overflow-checking
-  foreign-declarations emit-trace-info block-compilation line-number-database-size
+  debugging perform-lambda-lifting! disable-stack-overflow-checking no-c-syntax-checks
+  register-ffi-macro ffi-include-path-list ffi-dont-include
+  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
   target-heap-size target-stack-size target-heap-growth target-heap-shrinkage
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   target-initial-heap-size postponed-initforms
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
-  broken-constant-nodes inline-substitutions-enabled
+  broken-constant-nodes inline-substitutions-enabled enable-sharp-greater-read-syntax
   emit-profile profile-lambda-list profile-lambda-index profile-info-vector-name
   direct-call-ids foreign-type-table first-analysis emit-closure-info emit-line-info
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database scan-toplevel-assignments
@@ -61,7 +62,7 @@
   reorganize-recursive-bindings substitution-table simplify-named-call emit-unsafe-marker
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! source-filename compressed-literals literal-compression-threshold
-  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -81,10 +82,7 @@
 
 
 (include "tweaks")
-
-(define-constant default-profile-name "PROFILE")
-(define-constant default-inline-max-size 10)
-(define-constant funny-message-timeout 60000)
+(include "parameters")
 
 
 ;;; Compile a complete source file:
@@ -129,13 +127,13 @@
 	[hshrink (memq 'heap-shrinkage options)]
 	[kwstyle (memq 'keyword-style options)]
 	[lcthreshold (memq 'compress-literals options)]
-	[uses-units '()]
 	[uunit (memq 'unit options)]
 	[a-only (memq 'analyze-only options)]
 	[dynamic (memq 'dynamic options)]
 	[dumpnodes #f]
+	[ffi-mode (memq 'ffi options)]
+	[ffi-parse-mode (memq 'ffi-parse options)]
 	[quiet (memq 'quiet options)]
-	[start-time #f]
 	[ssize (or (memq 'nursery options) (memq 'stack-size options))] )
 
     (define (cputime) (##sys#fudge 6))
@@ -197,7 +195,18 @@
 	(printf "milliseconds needed for ~a: \t~s~%" pass (- (cputime) time0)) ) )
 
     (define (read-form in)
-      (##sys#read in infohook) )
+      (if (or ffi-mode ffi-parse-mode)
+	  (let ([s (read-string #f in)])
+	    (if (string=? s "")
+		#!eof
+		(cond [ffi-mode
+		       `(declare 
+			  (foreign-declare ,s)
+			  (foreign-parse ,s) ) ]
+		      [else
+		       `(declare
+			  (foreign-parse ,s) ) ] ) ) )
+	  (##sys#read in infohook) ) )
 
     (when uunit
       (set! unit-name (string->c-identifier (stringify (option-arg uunit)))) )
@@ -237,6 +246,7 @@
     (when (memq 'disable-interrupts options) (set! insert-timer-checks #f))
     (when (memq 'fixnum-arithmetic options) (set! number-type 'fixnum))
     (when (memq 'block options) (set! block-compilation #t))
+    (when (memq 'disable-c-syntax-checks options) (set! no-c-syntax-checks #t))
     (when (memq 'emit-external-prototypes-first options) (set! external-protos-first #t))
     (when (memq 'inline options) (set! inline-max-size default-inline-max-size))
     (when (memq 'track-scheme options) (set! emit-line-info #t))
@@ -268,13 +278,15 @@
 	      ipath) )
     (when (and outfile filename (string=? outfile filename))
       (quit "source- and output-filename are the same") )
-    (set! uses-units (map string->symbol (collect-options 'uses)))
-    (when (memq 'keep-shadowed-macros options)
-      (set! undefine-shadowed-macros #f) )
 
     ;; Handle feature options:
     (for-each register-feature! (collect-options 'feature))
 
+    ;; Handle FFI defines and include-paths:
+    (for-each register-ffi-macro (collect-options 'ffi-define))
+    (when (memq 'ffi-no-include options) (set! ffi-dont-include #t))
+    (set! ffi-include-path-list (append (collect-options 'ffi-include-path) ffi-include-path-list))
+
     ;; Load extensions:
     (set! ##sys#features (cons #:compiler-extension ##sys#features))
     (let ([extends (collect-options 'extend)])
@@ -303,12 +315,13 @@
 		     (else (quit "no filename available for `-extension' option")) ) ) ) ) ) ) )
 
     ;; Append required extensions to initforms:
-    (let ([ids (lset-difference eq? (map string->symbol (collect-options 'require-extension)) uses-units)])
+    (let ([ids (map string->symbol (collect-options 'require-extension))])
       (set! initforms
 	(append initforms (map (lambda (r) `(##core#require-extension ',r)) ids)) ) )
 
     (when (memq 'run-time-macros options)
       (set! ##sys#enable-runtime-macros #t) )
+    (enable-sharp-greater-read-syntax)
     (set! target-heap-size
       (if hsize
 	  (arg-val (option-arg hsize))
@@ -370,7 +383,6 @@
 	   (debugging 'r "debugging options" debugging-chicken)
 	   (debugging 'r "target heap size" target-heap-size)
 	   (debugging 'r "target stack size" target-stack-size)
-	   (set! start-time (cputime))
 
 	   ;; Read toplevel expressions:
 	   (set! ##sys#line-number-database (make-vector line-number-database-size '()))
@@ -392,10 +404,9 @@
 			     (append (map string->expr prelude)
 				     (reverse forms)
 				     (map string->expr postlude) ) ) )
-			(let* ((f (car files))
-			       (in (check-and-open-input-file f)) 
-			       (x1 (read-form in)) )
-			  (do ((x x1 (read-form in)))
+			(let* ([f (car files)]
+			       [in (check-and-open-input-file f)] )
+			  (do ([x (read-form in) (read-form in)])
 			      ((eof-object? x) 
 			       (close-checked-input-file in f) )
 			    (set! forms (cons x forms)) ) ) ) ] ) ) )
@@ -404,13 +415,13 @@
 	   (let ([proc (user-preprocessor-pass)])
 	     (when proc
 	       (when verbose (printf "User preprocessing pass...~%~!"))
-	       (set! forms (map proc forms))))
+	       (set! forms (cons (first forms) (map proc (cdr forms)))) ) )
 
 	   (print "source" '|1| forms)
 	   (begin-time)
-	   (unless (null? uses-units)
-	     (set! ##sys#explicit-library-modules (append ##sys#explicit-library-modules uses-units))
-	     (set! forms (cons `(declare (uses ,@uses-units)) forms)) )
+	   (let ([us (collect-options 'uses)])
+	     (unless (null? us)
+	       (set! forms (cons `(declare (uses ,@us)) forms)) ) )
 	   (let* ([exps0 (map canonicalize-expression (append initforms forms))]
 		  [pvec (gensym)]
 		  [plen (length profile-lambda-list)]
@@ -424,9 +435,10 @@
 				  ',(if unit-name #f profile-name))))
 			     '() )
 			 (map (lambda (pl)
-				`(##sys#set-profile-info-vector!
+				`(##core#inline 
+				  "C_i_setslot"
 				  ,profile-info-vector-name
-				  ',(car pl)
+				  ',(* profile-info-entry-size (car pl)) 
 				  ',(cdr pl) ) )
 			      profile-lambda-list)
 			 (let ([is (fold (lambda (clf r)
@@ -564,8 +576,6 @@
 			    (let ([node3 (perform-closure-conversion node2 db)])
 			      (end-time "closure conversion")
 			      (print-db "final-analysis" '|8| db i)
-			      (when (and ##sys#warnings-enabled (> (- (cputime) start-time) funny-message-timeout))
-				(display "(don't despair - still compiling...)\n") )
 			      (when export-file-name
 				(dump-exported-globals db export-file-name) )
 			      (let ([upap (user-post-analysis-pass)])
