@@ -1,6 +1,6 @@
 ;;; c-backend.scm - C-generating backend for the CHICKEN compiler
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -45,9 +45,9 @@
   foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home optimization-iterations debugging cleanup
-  file-io-only namespace-table
+  foreign-type-table-size file-io-only namespace-table
   unit-name insert-timer-checks used-units inlining external-variables
-  foreign-declarations emit-trace-info block-compilation line-number-database-size
+  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
   target-heap-size target-stack-size target-heap-growth target-heap-shrinkage
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   target-initial-heap-size disable-stack-overflow-checking
@@ -64,7 +64,7 @@
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub 
   expand-foreign-lambda*
   transform-direct-lambdas! target-include-file emit-unsafe-marker
-  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -485,8 +485,7 @@
 	    n) )
       (match (##sys#decode-seconds (current-seconds) #f)
 	[#(_ min hour mday mon year _ _ _ _)
-	  (gen "/* Generated from " source-file " by the CHICKEN compiler" #t
-	       "   http://www.call-with-current-continuation.org" #t
+	  (gen "/* Generated from " source-file " by the Chicken compiler" #t
 	       "   " (+ 1900 year) #\- (pad0 (add1 mon)) #\- (pad0 mday) #\space (pad0 hour) #\: (pad0 min) #t
 	       "   " (chicken-version #t) #t
 	       "   command line: ")
@@ -496,6 +495,9 @@
 		[else 
 		 (gen "   used units: ")
 		 (gen-list used-units) ] )
+	  (unless unit-name
+	    (gen #t "   default heap size: " default-default-target-heap-size #t)
+	    (gen "   default nursery (stack) size: " default-default-target-stack-size) )
 	  (gen #t "*/" #t #t "#include \"" target-include-file "\"")
 	  (when external-protos-first
 	    (generate-foreign-callback-stub-prototypes foreign-callback-stubs) )
