@@ -1,6 +1,6 @@
 ;;;; c-platform.scm - Platform specific parameters and definitions
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -42,9 +42,9 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home debugging
-  dump-nodes
+  foreign-type-table-size dump-nodes
   unit-name insert-timer-checks used-units inlining
-  foreign-declarations block-compilation line-number-database-size
+  foreign-declarations block-compilation analysis-database-size line-number-database-size
   target-heap-size target-stack-size 
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
@@ -56,7 +56,7 @@
   reorganize-recursive-bindings substitution-table simplify-named-call find-inlining-candidates perform-inlining!
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! decompose-lambda-list rewrite
-  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -122,13 +122,12 @@
 
 (define valid-compiler-options
   '(-help h help version verbose explicit-use quiet no-trace no-warnings unsafe block
-    check-syntax to-stdout no-usual-integrations case-insensitive no-lambda-info 
-    profile inline keep-shadowed-macros
+    check-syntax to-stdout no-usual-integrations case-insensitive no-lambda-info profile inline
     fixnum-arithmetic disable-interrupts optimize-leaf-routines check-imports
     lambda-lift run-time-macros tag-pointers accumulate-profile
     disable-stack-overflow-checks disable-c-syntax-checks unsafe-libraries raw 
     emit-external-prototypes-first track-scheme release
-    analyze-only dynamic extension) )
+    analyze-only dynamic ffi ffi-custom ffi-parse ffi-no-include extension) )
 
 (define valid-compiler-options-with-argument
   '(debug output-file include-path heap-size stack-size unit uses keyword-style require-extension 
@@ -162,14 +161,14 @@
 (define default-extended-bindings
   '(bitwise-and bitwise-ior bitwise-xor bitwise-not add1 sub1 fx+ fx- fx* fx/ fxmod
     fx= fx> fx< fx>= fx<= fixnum? fxneg fxmax fxmin identity fp+ fp- fp* fp/ fpmin fpmax fpneg
-    fp> fp< fp= fp>= fp<= fxand fxnot fxior fxxor fxshr fxshl bit-set?
+    fp> fp< fp= fp>= fp<= atom? fxand fxnot fxior fxxor fxshr fxshl
     arithmetic-shift void flush-output thread-specific thread-specific-set!
-    not-pair? atom? null-list? print print* error cpu-time proper-list? call/cc
+    not-pair? null-list? print print* error cpu-time proper-list? call/cc
     u8vector->byte-vector s8vector->byte-vector u16vector->byte-vector s16vector->byte-vector 
     u32vector->byte-vector 
     s32vector->byte-vector byte-vector-length block-ref block-set! number-of-slots
     f32vector->byte-vector f64vector->byte-vector byte-vector-ref byte-vector-set!
-    hash-table-ref any?
+    hash-table-ref
     first second third fourth make-record-instance
     u8vector-length s8vector-length u16vector-length s16vector-length u32vector-length s32vector-length
     f32vector-length f64vector-length setter
@@ -177,21 +176,21 @@
     f32vector-ref f64vector-ref
     u8vector-set! s8vector-set! u16vector-set! s16vector-set! u32vector-set! s32vector-set!
     locative-ref locative-set! locative->object locative? global-ref
-    null-pointer? pointer->object flonum? finite?) )
+    null-pointer? pointer->object) )
 
 (define internal-bindings
   '(##sys#slot ##sys#setslot ##sys#block-ref ##sys#block-set!
     ##sys#call-with-current-continuation ##sys#size ##sys#byte ##sys#setbyte
     ##sys#pointer? ##sys#generic-structure? ##sys#structure? ##sys#check-structure
     ##sys#check-exact ##sys#check-number ##sys#check-list ##sys#check-pair ##sys#check-string ##sys#check-symbol 
-    ##sys#check-char ##sys#check-vector ##sys#check-byte-vector ##sys#list ##sys#cons
+    ##sys#check-char ##sys#check-vector ##sys#check-byte-vector ##sys#list
     ##sys#call-with-values ##sys#fits-in-int? ##sys#fits-in-unsigned-int? ##sys#flonum-in-fixnum-range? 
     ##sys#fudge ##sys#immediate? ##sys#direct-return ##sys#context-switch
     ##sys#make-structure ##sys#apply ##sys#apply-values ##sys#continuation-graft
     ##sys#bytevector? ##sys#make-vector ##sys#setter
     ##sys#foreign-char-argument ##sys#foreign-fixnum-argument ##sys#foreign-flonum-argument
     ##sys#foreign-block-argument ##sys#foreign-number-vector-argument
-    ##sys#foreign-string-argument ##sys#foreign-pointer-argument ##sys#void
+    ##sys#foreign-string-argument ##sys#foreign-pointer-argument
     ##sys#foreign-integer-argument ##sys#foreign-unsigned-integer-argument ##sys#double->number
     ##sys#peek-fixnum ##sys#setislot ##sys#poke-integer ##sys#permanent? ##sys#values ##sys#poke-double
     ##sys#intern-symbol ##sys#make-symbol ##sys#null-pointer? ##sys#peek-byte) )
@@ -218,7 +217,7 @@
     f32vector->byte-vector f64vector->byte-vector ##sys#byte ##sys#setbyte byte-vector-ref byte-vector-set!
     u8vector-length s8vector-length u16vector-length s16vector-length u32vector-length s32vector-length
     f32vector-length f64vector-length ##sys#apply-values ##sys#setter setter
-    u8vector-ref s8vector-ref u16vector-ref s16vector-ref u32vector-ref s32vector-ref
+    u8vector-ref s8vector-ref u16vector-ref s16vector-ref
     u8vector-set! s8vector-set! u16vector-set! s16vector-set! u32vector-set! s32vector-set!
     ##sys#intern-symbol ##sys#make-symbol make-record-instance error cpu-time ##sys#block-set!) )
 
@@ -497,23 +496,21 @@
 	       (and-let* ((sym (car (node-parameters arg2)))
 			  (val (get db sym 'value)) )
 		 (and (eq? '##core#lambda (node-class val))
-		      (let ((llist (third (node-parameters val))))
-			(and (proper-list? llist)
-			     (= 2 (length (third (node-parameters val))))
-			     (let ((tmp (gensym))
-				   (tmpk (gensym 'r)) )
-			       (debugging 'o "removing single-valued `call-with-values'" (node-parameters val))
-			       (make-node
-				'let (list tmp)
+		      (= 2 (length (third (node-parameters val))))
+		      (let ((tmp (gensym))
+			    (tmpk (gensym 'r)) )
+			(debugging 'x "removing single-valued `call-with-values'" (node-parameters val))
+			(make-node
+			 'let (list tmp)
+			 (list (make-node
+				'##core#lambda
+				(list (gensym 'f_) #f (list tmpk) 0)
 				(list (make-node
-				       '##core#lambda
-				       (list (gensym 'f_) #f (list tmpk) 0)
-				       (list (make-node
-					      '##core#call '(#t)
-					      (list arg2 cont (varnode tmpk)) ) ) ) 
-				      (make-node
 				       '##core#call '(#t)
-				       (list arg1 (varnode tmp)) ) ) ) ) ) ) ) ) ) ) ) )
+				       (list arg2 cont (varnode tmpk)) ) ) ) 
+			       (make-node
+				'##core#call '(#t)
+				(list arg1 (varnode tmp)) ) ) ) ) ) ) ) ) ) )
   (rewrite 'call-with-values 8 rewrite-c-w-v)
   (rewrite '##sys#call-with-values 8 rewrite-c-w-v) )
 
@@ -574,6 +571,7 @@
 (rewrite 'symbol? 2 1 "C_i_symbolp" #t #f)
 (rewrite 'vector? 2 1 "C_i_vectorp" #t #f)
 (rewrite 'pair? 2 1 "C_i_pairp" #t "C_notvemptyp")
+(rewrite 'atom? 2 1 "C_i_atomp" #t #f)
 (rewrite 'procedure? 2 1 "C_i_closurep" #t #f)
 (rewrite 'port? 2 1 "C_i_portp" #t #f)
 (rewrite 'boolean? 2 1 "C_booleanp" #t #f)
@@ -582,9 +580,7 @@
 (rewrite 'rational? 2 1 "C_i_numberp" #t #f)
 (rewrite 'real? 2 1 "C_i_numberp" #t #f)
 (rewrite 'integer? 2 1 "C_i_integerp" #t #f)
-(rewrite 'flonum? 2 1 "C_i_flonump" #t #f)
 (rewrite 'fixnum? 2 1 "C_fixnump" #t #f)
-(rewrite 'finite? 2 1 "C_i_finitep" #f #f)
 (rewrite '##sys#pointer? 2 1 "C_pointerp" #t #f)
 (rewrite '##sys#generic-structure? 2 1 "C_structurep" #t #f)
 (rewrite 'exact? 2 1 "C_fixnump" #f #f)
@@ -907,9 +903,6 @@
 (rewrite 'u16vector-ref 2 2 "C_u_i_u16vector_ref" #f #f)
 (rewrite 's16vector-ref 2 2 "C_u_i_s16vector_ref" #f #f)
 
-(rewrite 'u32vector-ref 22 2 "C_a_i_u32vector_ref" #f words-per-flonum "C_u_i_u32vector_ref")
-(rewrite 's32vector-ref 22 2 "C_a_i_s32vector_ref" #f words-per-flonum "C_u_i_s32vector_ref")
-
 (rewrite 'u8vector-set! 2 3 "C_u_i_u8vector_set" #f #f)
 (rewrite 's8vector-set! 2 3 "C_u_i_s8vector_set" #f #f)
 (rewrite 'u16vector-set! 2 3 "C_u_i_u16vector_set" #f #f)
@@ -927,7 +920,6 @@
 (rewrite 'f64vector-length 2 1 "C_u_i_64vector_length" #f #f)
 
 (rewrite 'not-pair? 17 1 "C_i_not_pair_p")
-(rewrite 'atom? 17 1 "C_i_not_pair_p")
 (rewrite 'null-list? 17 1 "C_i_null_list_p" "C_i_nullp")
 
 (rewrite 'u8vector->byte-vector 7 1 "C_slot" 1 #f)
@@ -1037,32 +1029,3 @@
 			 '##core#call '(#t)
 			 (list cont (varnode (cdr a))) ) ) ) ) ) ) ) ) )
 			       
-(rewrite 'void 3 '##sys#undefined-value)
-(rewrite '##sys#void 3 '##sys#undefined-value)
-
-(rewrite
- 'any? 8
- (lambda (db classargs cont callargs) 
-   (and (= 1 (length callargs))
-	(let ((arg (car callargs)))
-	  (make-node
-	   '##core#call '(#t) 
-	   (list cont
-		 (if (and (eq? '##core#variable (node-class arg))
-			  (not (get db (car (node-parameters arg)) 'global)) )
-		     (qnode #t)
-		     (make-node 
-		      '##core#inline '("C_anyp")
-		      (list arg)) ) ) ) ) ) ) )
-
-(rewrite
- 'bit-set? 8
- (lambda (db classargs cont callargs)
-   (and (= 2 (length callargs))
-	(make-node
-	 '##core#call '(#t)
-	 (list cont
-	       (make-node
-		'##core#inline 
-		(list (if (eq? number-type 'fixnum) "C_u_i_bit_setp" "C_i_bit_setp"))
-		callargs) ) ) ) ) )
