@@ -1,6 +1,6 @@
 ;;;; chicken-ffi-macros.scm
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -45,26 +45,26 @@
   `(##core#define-foreign-variable ,@(map (lambda (x) (list 'quote x)) xs)) )
 
 (define-macro (foreign-lambda . xs)
-  (##sys#check-syntax 'foreign-lambda xs `#(_ 1)) 
+  (##sys#check-syntax 'foreign-lambda xs '(_ _ . #(_ 0)))
   `(##core#foreign-lambda ,@(map (lambda (x) (list 'quote x)) xs)) )
 
 (define-macro (foreign-lambda* . xs)
-  (##sys#check-syntax 'foreign-lambda* xs `#(_ 1)) 
+  (##sys#check-syntax 'foreign-lambda* xs '(_ #((_ _) 0) . #(string 1)))
   `(##core#foreign-lambda* ,@(map (lambda (x) (list 'quote x)) xs)) )
 
 (define-macro (foreign-safe-lambda . xs)
-  (##sys#check-syntax 'foreign-safe-lambda xs `#(_ 1)) 
+  (##sys#check-syntax 'foreign-safe-lambda xs '(_ _ . #(_ 0)))
   `(##core#foreign-callback-lambda ,@(map (lambda (x) (list 'quote x)) xs)) )
 
 (define-macro (foreign-safe-lambda* . xs)
-  (##sys#check-syntax 'foreign-safe-lambda* xs `#(_ 1)) 
+  (##sys#check-syntax 'foreign-safe-lambda* xs '(_ #((_ _) 0) . #(string 1)))
   `(##core#foreign-callback-lambda* ,@(map (lambda (x) (list 'quote x)) xs)) )
 
 (define-macro (foreign-primitive . xs)
-  (##sys#check-syntax 'foreign-primitive xs `#(_ 1)) 
+  (##sys#check-syntax 'foreign-primitive xs '#(_ 1)) 
   `(##core#foreign-primitive ,@(map (lambda (x) (list 'quote x)) xs)) )
 
-(define-macro (foreign-safe-wrapper . form)
+(define-macro (foreign-safe-wrapper . form) ; DEPRECATED
   (##sys#check-syntax 'foreign-safe-wrapper form '(_ string . #(_ 1)))
   (if (string? (caddr form))
       (begin
@@ -181,14 +181,13 @@
     (define (stype type)
       (cond [(not (pair? type)) type]
 	    [(eq? 'const (car type)) (stype (cadr type))]
-	    [(memq (car type) '(struct union)) `(nonnull-c-pointer ,type)]
+	    [(memq (car type) '(struct union)) `(c-pointer ,type)]
 	    [else type] ) )
     (define (strtype type)
       (or (eq? type tname)
 	  (and (pair? type)
 	       (or (and (eq? 'const (car type)) (strtype (cadr type)))
 		   (memq (car type) '(struct union)) ) ) ) ) ; handle instances?
-    (##sys#hash-table-set! ##compiler#foreign-type-table tname `(nonnull-c-pointer ,fname))
     (do ((slts slots (cdr slts)))
 	((or (null? slts) (not (pair? (car slts))) (not (keyword? (caar slts))))
 	 (set! slots slts) )
@@ -197,6 +196,7 @@
 	((constructor:) (set! ctor (cadar slts)))
 	((destructor:) (set! dtor (cadar slts)))
 	(else (syntax-error 'define-foreign-record "invalid foreign record-type specification" (car slts))) ) )
+    (##sys#hash-table-set! ##compiler#foreign-type-table tname `(c-pointer ,fname))
     `(begin
        ,@(if (pair? name)
 	     '() 
@@ -296,6 +296,95 @@
   `(##core#declare '(foreign-declare ,@strs) '(foreign-parse ,@strs)) )
 
 
+;;; Foreign enumerations (or enum-like constants)
+
+(define-macro (define-foreign-enum typename . enums)
+  (let ((name typename)
+	(type (->string typename)) )
+    (when (and (list? typename) (= 2 (length typename)))
+      (set! name (car typename))
+      (set! type (cadr typename)) )
+    (let* ((symbols (map (lambda (e) (if (pair? e) (car e) e)) enums))
+	   (aliases (map gensym symbols)) 
+	   (vals (map (lambda (e) (if (pair? e) (cadr e) e)) enums))
+	   (s->e (string->symbol (conc name "->number")))
+	   (e->s (string->symbol (conc "number->" name)) ) )
+      `(begin
+	 ,@(map (lambda (a v) `(define-foreign-variable ,a integer ,(->string v))) aliases vals)
+	 (define (,s->e syms)
+	   (let loop ((syms (if (symbol? syms) (list syms) syms)) (sum 0))
+	     (if (null? syms) 
+		 sum
+		 (loop
+		  (cdr syms)
+		  (bitwise-ior
+		   sum
+		   (let ((val (car syms)))
+		     (case val
+		       ,@(map (lambda (a s) `((,s) ,a)) aliases symbols)
+		       (else (error "not a member of enum" val ',name)) ) ) ) ) ) ) )
+	 (define (,e->s val)
+	   (cond 
+	    ,@(map (lambda (a s) `((= val ,a) ',s)) aliases symbols)
+	    (else '()) ) )
+	 (define-foreign-type ,name ,type ,s->e ,e->s) ) ) ) )
+
+
+;;; The dollar macro
+
+(define-macro ($ func . args)
+  (define (conv arg err)
+    (cond ((fixnum? arg) `(int ,arg))
+	  ((number? arg) `(double ,arg))
+	  ((string? arg) `(nonnull-c-string ,arg))
+	  ((char? arg) `(char ,arg))
+	  ((boolean? arg) `(bool ,arg))
+	  (else (syntax-error '$ "argument has unknown type" arg)) ) )
+  (let* ((rtype (cond ((and (pair? args) (symbol? (car args)))
+		       (let ((rtype func))
+			 (set! func (car args))
+			 (set! args (cdr args))
+			 rtype) ) 
+		      (else 'void)))
+	 (args (map (lambda (arg)
+		      (cond ((atom? arg) (conv arg #t))
+			    ((list? arg)
+			     (case (car arg)
+			       ((quote)
+				(if (pair? (cdr arg))
+				    (let ((val (cadr arg)))
+				      (cond ((symbol? val) `(symbol ',val))
+					    ((u8vector? val) `(nonnull-u8vector ',val))
+					    ((s8vector? val) `(nonnull-s8vector ',val))
+					    ((u16vector? val) `(nonnull-u16vector ',val))
+					    ((s16vector? val) `(nonnull-s16vector ',val))
+					    ((u32vector? val) `(nonnull-u32vector ',val))
+					    ((s32vector? val) `(nonnull-s32vector ',val))
+					    ((f32vector? val) `(nonnull-f32vector ',val))
+					    ((f64vector? val) `(nonnull-f64vector ',val))
+					    ((or (pair? val) (vector? val)) `(scheme-object ',val))
+					    (else (conv val #f)) ) )
+				    arg) )
+			       ((location) `(nonnull-c-pointer ,arg))
+			       (else arg) ) )
+			    (else (syntax-error '$ "argument has unknown type" arg)) ) )
+		    args) )
+	 (syms (map (lambda (_) (gensym)) args)))
+    (if (null? args)
+	(if (eq? 'void rtype)
+	    `(foreign-code ,(conc func "();"))
+	    `(foreign-value ,(conc func "()") ,rtype) )
+	`((foreign-lambda* ,rtype 
+	      ,(map (lambda (arg sym)
+		      (list (car arg) sym)) 
+		    args syms)
+	    ,(let ((body (conc func "(" (string-intersperse (map ->string syms) ",") ")")))
+	       (if (eq? rtype 'void) 
+		   (string-append body ";")
+		   (string-append "return(" body ");") ) ) )
+	  ,@(map cadr args) ) ) ) )
+
+
 ;;; Deprecated FFI macros
 
 (define-macro (define-deprecated-macro old new)
