@@ -1,6 +1,6 @@
 ;;;; chicken-more-macros.scm - More syntax extensions
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -346,7 +346,7 @@
    (##sys#check-syntax 'define-constant form '(symbol _))
    `(##core#define-constant ',(car form) ,(cadr form)) ) )
 
-(##sys#register-macro-2
+(##sys#register-macro-2			; DEPRECATED
  'critical-section
  (lambda (form)
    `(##sys#dynamic-wind
@@ -382,7 +382,7 @@
        (##sys#error "syntax error in `cond-expand' form" x (cons 'cond-expand clauses)) )
 
      (define (test fx)
-       (cond ((symbol? fx) (##sys#test-feature fx))
+       (cond ((symbol? fx) (##sys#feature? fx))
 	     ((not (pair? fx)) (err fx))
 	     (else
 	      (let ((rest (##sys#slot fx 1)))
@@ -708,44 +708,6 @@
 	 `(##sys#register-record-printer ',head ,@body) ] ) )
 
 
-;;; TinyCLOS:
-
-(define-macro (define-class name supers slots . meta)
-  (##sys#check-syntax 'define-class name 'symbol)
-  (##sys#check-syntax 'define-class supers '#(_ 0))
-  (##sys#check-syntax 'define-class slots '#(_ 0))
-  (##sys#check-syntax 'define-class meta '#(_ 0 1))
-  `(##core#set! ,name
-     (make ,(if (pair? meta) (##sys#slot meta 0) '<class>)
-       'name ',name
-       'direct-supers (list ,@(if (null? supers) '(<object>) supers))
-       'direct-slots (list ,@(map (lambda (s) `',s) slots)) ) ) )
-
-(define-macro (define-generic name . class)
-  (##sys#check-syntax 'define-generic name 'symbol)
-  `(define ,name (make ,(:optional class '<generic>) 'name ',name)) )
-
-(define-macro (define-method head . body)
-  (##sys#check-syntax 'define-method head '(symbol . _))
-  (##sys#check-syntax 'define-method body '#(_ 1))
-  (let gather ([args (##sys#slot head 1)]
-	       [specs '()]
-	       [vars '()] )
-    (if (or (not (pair? args)) 
-	    (memq (car args) '(#!optional #!key #!rest)) )
-	(let ([name (##sys#slot head 0)])
-	  `(##core#set! ,name
-			(##tinyclos#add-global-method
-			 (##core#global-ref ,name)
-			 ',name
-			 (list ,@(reverse specs))
-			 (##core#named-lambda ,name (call-next-method ,@(reverse vars) ,@args) ,@body) ) ) )
-	(let ([arg (##sys#slot args 0)])
-	  (gather (##sys#slot args 1)
-		  (cons (if (pair? arg) (cadr arg) '<top>) specs)
-		  (cons (if (pair? arg) (car arg) arg) vars) ) ) ) ) )
-
-
 ;;; Exceptions:
 
 (define-macro (handle-exceptions var handler . body)
@@ -919,9 +881,6 @@
       `(letrec ((,(car head) (lambda ,(cdr head) ,@args))) ,(car head))
       `(letrec ((,head ,@args)) ,head)))
 
-(eval-when (compile load eval)
-  (register-feature! 'srfi-8 'srfi-16 'srfi-26 'srfi-31 'srfi-15 'srfi-11) )
-
 
 ;;; Definitions available at macroexpansion-time:
 
@@ -936,3 +895,8 @@
 	`(define ,name ,body)
 	'(begin) ) ) )
 
+
+;;;; Register features provided by this file
+
+(eval-when (compile load eval)
+  (register-feature! 'srfi-8 'srfi-16 'srfi-26 'srfi-31 'srfi-15 'srfi-11) )
