@@ -1,6 +1,6 @@
 ;;;; chicken-profile.scm - Formatted display of profile outputs - felix -*- Scheme -*-
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -46,14 +46,12 @@
   (display #<<EOF
 Usage: chicken-profile [FILENAME | OPTION] ...
 
- -sort-by-calls            sort output by call frequency
- -sort-by-time             sort output by procedure execution time
- -sort-by-avg              sort output by average procedure execution time
- -sort-by-name             sort output alphabetically by procedure name
- -no-unused                remove procedures that are never called
- -help                     show this text and exit
- -version                  show version and exit
- -release                  show release number and exit
+ -sort-by-calls            Sort output by call frequency
+ -sort-by-time             Sort output by procedure execution time
+ -sort-by-avg              Sort output by average procedure execution time
+ -sort-by-name             Sort output alphabetically by procedure name
+ -no-unused                Remove procedures that are never called
+ -help                     Show this text
 
  FILENAME defaults to "PROFILE"
 
@@ -71,12 +69,6 @@ EOF
 	      [rest (cdr args)] )
 	  (match arg
 	    [(or "-h" "-help" "--help") (print-usage)]
-	    [(or "-v" "-version") 
-	     (print "chicken-profile - Version " (chicken-version))
-	     (exit) ]
-	    ["-release" 
-	     (print (chicken-version))
-	     (exit) ]
 	    ["-no-unused" (set! no-unused #t)]
 	    ["-sort-by-calls" (set! sort-by sort-by-calls)]
 	    ["-sort-by-time" (set! sort-by sort-by-time)]
