@@ -1,6 +1,6 @@
 ;;;; chicken-setup
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -35,26 +35,28 @@
 
 (declare
   (run-time-macros)
-  (uses srfi-1 regex utils posix tcp match srfi-18 srfi-13)
-  (export move-file run:execute make/proc uninstall-extension
-	  install-extension install-program install-script
-	  setup-verbose-flag setup-install-flag installation-prefix
-	  find-library find-header program-path remove-file* patch
-	  yes-or-no? setup-build-directory setup-root-directory
-	  create-directory test-compile try-compile copy-file run-verbose
-	  required-chicken-version required-extension-version
-	  cross-chicken) )
+  (uses srfi-1 regex utils posix tcp match srfi-18)
+  (export move-file run:execute make/proc uninstall-extension install-extension install-program install-script
+	  setup-verbose-flag setup-install-flag installation-prefix find-library find-header
+	  program-path remove-file* patch yes-or-no? setup-build-directory setup-root-directory create-directory
+	  test-compile copy-file run-verbose) )
 
-#>
-#ifndef C_USE_C_DEFAULTS
+
+(foreign-declare #<<EOF
+#ifdef CMAKE_BUILD
+# include "chicken-paths.h"
+# include "stack-size.h"
+#elif defined(C_USE_C_DEFAULTS)
+# include "chicken-defaults.h"
+#else
 # define C_INSTALL_BIN_HOME   NULL
 # ifdef _MSC_VER
-#  define C_INSTALL_CC                "cl"
+#  define C_CC                "cl"
 # else
 #  ifdef __GNUC__
-#   define C_INSTALL_CC                "gcc"
+#   define C_CC                "gcc"
 #  else
-#   define C_INSTALL_CC                "cc"
+#   define C_CC                "cc"
 #  endif
 # endif
 #endif
@@ -70,52 +72,31 @@ static void create_directory(char *pathname)
 #else
 static void create_directory(char *pathname) {}
 #endif
-
-#ifndef C_TARGET_CC
-# define C_TARGET_CC  C_INSTALL_CC
-#endif
-
-#ifndef C_TARGET_CXX
-# define C_TARGET_CXX  C_INSTALL_CXX
-#endif
-
-#ifndef C_TARGET_CFLAGS
-# define C_TARGET_CFLAGS  ""
-#endif
-
-#ifndef C_TARGET_LFLAGS
-# define C_TARGET_LFLAGS  ""
-#endif
-<#
-
-
-(define-constant setup-file-extension "setup-info")
-(define-constant remote-repository-name "repository")
-(define-constant installed-executables '("chicken" "csc" "csi" "chicken-setup" "chicken-profile"))
+EOF
+)
 
 
+(include "parameters.scm")
+(include "build.scm")
 (include "chicken-more-macros.scm")
 
 
 (define-constant long-options
   '("-help" "-uninstall" "-list" "-run" "-repository" "-program-path" "-version" "-script" "-check"
     "-fetch" "-host" "-proxy" "-keep" "-verbose" "-csc-option" "-dont-ask" "-no-install" "-docindex" "-eval"
-    "-debug" "-ls" "-release" "-test" "-fetch-tree" "-tree" "-svn" "-local" "-destdir" "-revision") )
+    "-debug") )
 
 (define-constant short-options
-  '(#\h #\u #\l #\r #\R #\P #\V #\s #\C #\f #\H #\p #\k #\v #\c #\d #\n #\i #\e #\D #f #f #\t #f #f #f #f #f #f) )
+  '(#\h #\u #\l #\r #\R #\P #\V #\s #\C #\f #\H #\p #\k #\v #\c #\d #\n #\i #\e #\D) )
 
 
 (define *install-bin-path* 
-  (or (and-let* ((p (getenv "CHICKEN_PREFIX")))
+  (or (and-let* ((p (getenv prefix-environment-variable)))
 	(make-pathname p "bin") )
-      (getenv "CHICKEN_HOME")
+      (getenv home-environment-variable) 
       (foreign-value "C_INSTALL_BIN_HOME" c-string) ) )
 
-(define *cc* (foreign-value "C_TARGET_CC" c-string))
-(define *cxx* (foreign-value "C_TARGET_CXX" c-string))
-(define *target-cflags* (foreign-value "C_TARGET_CFLAGS" c-string))
-(define *target-lflags* (foreign-value "C_TARGET_LFLAGS" c-string))
+(define *cc* (foreign-value "C_CC" c-string))
 
 (define *windows*
   (and (eq? (software-type) 'windows) 
@@ -131,8 +112,6 @@ static void create_directory(char *pathname) {}
 (define *remove-command* (if *windows-shell* "del /Q /S" "rm -fr"))
 (define *move-command* (if *windows-shell* 'move 'mv))
 
-(define (cross-chicken) (##sys#fudge 39))
-
 (define create-directory
   (let ()
     (define (verb dir)
@@ -143,15 +122,10 @@ static void create_directory(char *pathname) {}
 	  ((foreign-lambda void "create_directory" c-string) dir) ) 
 	(lambda (dir)
 	  (verb dir)
-	  (system* "mkdir -p ~a" (quotewrap dir) ) ) ) ) )
-
-(define (quotewrap str)
-  (if (string-any char-whitespace? str)
-      (string-append "\"" str "\"") 
-      str) )
+	  (system* "mkdir -p ~a" dir) ) ) ) )
 
 (define setup-root-directory (make-parameter #f))
-(define setup-build-directory (make-parameter (current-directory)))
+(define setup-build-directory (make-parameter #f))
 (define setup-verbose-flag (make-parameter #f))
 (define setup-install-flag (make-parameter #t))
 
@@ -160,7 +134,7 @@ static void create_directory(char *pathname) {}
 (define *tmpdir-created* #f)
 (define *keep-stuff* #f)
 (define *csc-options* '())
-(define *abort-hook* #f)
+(define *abort-yes-or-no* #f)
 (define *dont-ask* #f)
 (define *rebuild-doc-index* #f)
 (define *check-repository* #f)
@@ -170,17 +144,13 @@ static void create_directory(char *pathname) {}
 (define *proxy-port* #f)
 (define *example-directory* (make-pathname (chicken-home) "examples"))
 (define *base-directory* (current-directory))
-(define *fetch-tree-only* #f)
-(define *svn-repository* #f)
-(define *local-repository* #f)
-(define *destdir* #f)
-(define *repository-hosts* '(("www.call-with-current-continuation.org" "eggs" 80)))
-(define *revision* #f)
-(define *repository-tree-downloaded* #f)
 
 
-(define (abort-setup)
-  (*abort-hook* #f) )
+; Repository-format:
+;
+; ((NAME FILENAME REQUIRED-NAME ...) ...)
+(define repository-hosts '(("www.call-with-current-continuation.org" "eggs" 80)))
+
 
 (define (yes-or-no? str . default)
   (let ((def (:optional default #f)))
@@ -193,7 +163,7 @@ static void create_directory(char *pathname) {}
 	      ((and def (string=? "" ln)) (set! ln def)) )
 	(cond ((string-ci=? "yes" ln) #t)
 	      ((string-ci=? "no" ln) #f)
-	      ((string-ci=? "abort" ln) (abort-setup))
+	      ((string-ci=? "abort" ln) (*abort-yes-or-no* #f))
 	      (else
 	       (printf "~%Please enter \"yes\", \"no\" or \"abort\".~%")
 	       (loop) ) ) ) ) ) )
@@ -214,18 +184,18 @@ static void create_directory(char *pathname) {}
     (both
      (let ((tmp (create-temporary-file)))
        (patch (list both tmp) rx subst)
-       (system* "~A ~A ~A" *move-command* (quotewrap tmp) (quotewrap both) ) ) ) ) )
+       (system* "~A ~A ~A" *move-command* tmp both) ) ) ) )
 
 (define run-verbose (make-parameter #t))
 
 (define (fixpath prg)
   (cond ((string=? prg "csc")
 	 (string-intersperse 
-	  (cons* (quotewrap (make-pathname *install-bin-path* prg))
+	  (cons* (make-pathname *install-bin-path* prg)
 		 "-feature" "compiling-extension"
 		 *csc-options*) 
 	  " ") )
-	((member prg installed-executables) (quotewrap (make-pathname *install-bin-path* prg)))
+	((member prg installed-executables) (make-pathname *install-bin-path* prg))
 	(else prg) ) )
 
 (define (fixmaketarget file)
@@ -234,6 +204,9 @@ static void create_directory(char *pathname) {}
       (pathname-replace-extension file ##sys#load-dynamic-extension)
       file) )
 
+(define (unformatify str)
+  (string-translate* str '(("~" . "~~"))) )
+
 (define (run:execute explist)
   (define (smooth lst)
     (let ((slst (map ->string lst)))
@@ -241,7 +214,7 @@ static void create_directory(char *pathname) {}
   (for-each
    (lambda (cmd)
      (when (run-verbose) (printf "  ~A~%~!" cmd))
-     (system* "~a" cmd) )
+     (system* (unformatify cmd) ))
    (map smooth explist) ) )
 
 (define-macro (run . explist)
@@ -291,11 +264,11 @@ static void create_directory(char *pathname) {}
 
 (define (make:check-argv argv)
   (or (string? argv)
-      (andmap string? argv)
-      (error "argument is not a string or string list" argv)))
+      (and (vector? argv)
+	   (andmap string? (vector->list argv)))
+      (error "argument is not a string or string vector" argv)))
 
 (define (make:make/proc/helper spec argv)
-  (when (vector? argv) (set! argv (vector->list argv)))
   (make:check-spec spec)
   (make:check-argv argv)
   (letrec ((made '())
@@ -307,8 +280,10 @@ static void create_directory(char *pathname) {}
 		     (s2 (fixmaketarget s)) 
 		     (date (and (file-exists? s2)
 				(file-modification-time s2))))
+
 		(when (setup-verbose-flag)
 		  (printf "make: ~achecking ~a~%" indent s2))
+
 		(if line
 		    (let ((deps (cadr line)))
 		      (for-each (let ((new-indent (string-append " " indent)))
@@ -352,8 +327,8 @@ static void create_directory(char *pathname) {}
 		      (error (sprintf "don't know how to make ~a" s2))))))))
     (cond
      ((string? argv) (make-file argv ""))
-     ((null? argv) (make-file (caar spec) ""))
-     (else (for-each (lambda (f) (make-file f "")) argv)))
+     ((equal? argv '#()) (make-file (caar spec) ""))
+     (else (for-each (lambda (f) (make-file f "")) (vector->list argv))))
     (when (setup-verbose-flag)
       (for-each (lambda (item)
 		  (printf "make: made ~a~%" item))
@@ -361,40 +336,43 @@ static void create_directory(char *pathname) {}
 
 (define make/proc
   (case-lambda
-   ((spec) (make:make/proc/helper spec '()))
+   ((spec) (make:make/proc/helper spec '#()))
    ((spec argv) (make:make/proc/helper spec argv))))
 
-(define-macro (make spec #!optional (argv '()))
-  (let ((form-error (lambda (s . p) (apply error s spec p))))
-    (and (or (list? spec) (form-error "illegal specification (not a sequence)"))
-	 (or (pair? spec) (form-error "empty specification"))
-	 (andmap
-	  (lambda (line)
-	    (and (or (and (list? line) (>= (length line) 2))
-		     (form-error "clause does not have at least 2 parts" line))
-		 (let ((name (car line)))
-		   (or (list? (cadr line))
-		       (make:line-error "second part of clause is not a sequence" (cadr line) name)))))
-	  spec))
-    `(make/proc (list ,@(map (lambda (line)
-			       `(list ,(car line)
-				      (list ,@(cadr line))
-				      ,@(let ((l (cddr line)))
-					  (if (null? l)
-					      '()
-					      `((lambda ()
-						  ,@l))))))
-			     spec))
-		,argv)))
+(define-macro (make spec . argv)
+  (let ((make (lambda (spec argv)
+		(let ((form-error (lambda (s . p) (apply error s spec p))))
+		  (and (or (list? spec) (form-error "illegal specification (not a sequence)"))
+		       (or (pair? spec) (form-error "empty specification"))
+		       (andmap
+			(lambda (line)
+			  (and (or (and (list? line) (>= (length line) 2))
+				   (form-error "clause does not have at least 2 parts" line))
+			       (let ((name (car line)))
+				 (or (list? (cadr line))
+				     (make:line-error "second part of clause is not a sequence" (cadr line) name)))))
+			spec))
+		  `(make/proc (list ,@(map (lambda (line)
+					     `(list ,(car line)
+						    (list ,@(cadr line))
+						    ,@(let ((l (cddr line)))
+							(if (null? l)
+							    '()
+							    `((lambda ()
+								,@l))))))
+					   spec))
+			      ,(if (vector? argv) `',argv (car argv)))))))
+    (if (pair? argv)
+	(make spec argv)
+	(make spec '#())) ) )
 
 (define (usage)
   (display #<<EOF
 usage: chicken-setup [OPTION ...] FILENAME
 
-  -h  -help                      show this text and exit
-  -V  -version                   show version of this program and exit
-      -release                   show release number and exit
-  -R  -repository [PATH]         prints the location of the extension repository
+  -h  -help                      show this text
+  -V  -version                   show version of this program
+  -R  -repository                prints the location of the extension repository
   -u  -uninstall                 remove the following extension from repository
   -H  -host HOSTNAME[:PORT]      specify alternative host for downloading
   -p  -proxy HOSTNAME[:PORT]     connect via proxy
@@ -411,14 +389,6 @@ usage: chicken-setup [OPTION ...] FILENAME
   -i  -docindex                  display path for documentation index
   -C  -check                     check for available upgrades
   -e  -eval EXPRESSION           evaluate expression
-  -t  -test EXTENSION ...        return success if all given extensions are installed
-      -ls EXTENSION              list installed files for extension
-      -fetch-tree                download and show repository catalog
-      -tree FILENAME             use repository catalog from given file
-      -svn URL                   fetch extension from subversion repository
-      -local PATH                fetch extension from local filesystem
-      -destdir PATH              specify alternative installation prefix
-      -revision REV              specify SVN revision for checkout
   --                             ignore all following arguments
 
   Builds and installs extension libraries.
@@ -427,14 +397,11 @@ EOF
   )
   (exit) )
 
-(define (make-setup-info-pathname fn #!optional (rpath (repository-path)))
-  (make-pathname rpath fn setup-file-extension) )
-
 (define program-path (make-parameter *install-bin-path*))
 
 (define installation-prefix
   (make-parameter
-   (or (getenv "CHICKEN_PREFIX")
+   (or (getenv prefix-environment-variable) 
        (match (string-match "(.*)/bin/?" *install-bin-path*)
 	 ((_ p) p)
 	 (_ #f) ) ) ) )
@@ -461,12 +428,9 @@ EOF
 		      '() ) 
 		,@info)) )
       (when (setup-verbose-flag) (printf "writing info ~A -> ~S ...~%" id info))
-      (let* ((sid (->string id))
-	    (setup-file (make-setup-info-pathname sid (repo-path #t)))
-	    (write-setup-info (with-output-to-file setup-file
-				(cut pp info))))
-	(unless *windows-shell* (run (chmod a+r ,setup-file)))
-	write-setup-info))))
+      (let ((sid (->string id)))
+	(with-output-to-file (make-pathname (repo-path) sid setup-file-extension)
+	  (cut pp info) ) ) ) ) )
 
 (define (fix-exports id info)
   (let-values (((einfo oinfo) (partition (lambda (item) (eq? 'exports (car item))) info)))
@@ -489,60 +453,62 @@ EOF
 	  (values `((exports ,@exports)) oinfo)
 	  (values '() oinfo) ) ) ) )
 
+(define (simple-install filename)
+  (let ((so (pathname-replace-extension filename ##sys#load-dynamic-extension))
+	(id (pathname-strip-extension filename)) )
+    (run (csc -O2 -no-trace -s ,filename -emit-exports ,(make-pathname #f id "exports")))
+    (when (setup-install-flag)
+      (unless *windows* (run (,*remove-command* ,(make-pathname (repo-path) so))))
+      (run (,*copy-command* ,so ,(repo-path)))
+      (write-info id (list (make-pathname (repository-path) so)) '()) )
+    (unless *keep-stuff* (run (rm ,so)) ) ) )
+
 (define (compute-tmpdir fname)
-  (if (equal? "egg-dir" (pathname-extension fname))
-      fname
-      (string-append fname "-dir") ) )
+  (string-append fname ".dir") )
 
 (define (chdir dir)
   (when (setup-verbose-flag) (printf "changing working directory to `~A'~%" dir))
   (change-directory dir) )
 
 (define (rmtmpdir)
-  (unless (string=? (current-directory) *base-directory*)
-    (chdir *base-directory*) )
+  (chdir *base-directory*)
   (when *tmpdir-created*
     (set! *tmpdir-created* #f)
     (unless *keep-stuff*
       (when (setup-verbose-flag) (printf "removing temporary directory `~A'~%" *temporary-directory*))
-      (handle-exceptions ex
-	  (warning "removal of temporary directory failed" *temporary-directory*)
-	(run (,*remove-command* ,(quotewrap *temporary-directory*))) )) ) )
+      (run (,*remove-command* ,*temporary-directory*))) ))
 
-(define (unpack/enter filename)
+(define (unpack filename)
   (define (testgz fn)
     (with-input-from-file fn
       (lambda () (string=? "\x1f\x8b" (read-string 2))) ) )
   (let ((tmpdir (compute-tmpdir filename)))
-    (cond ((file-exists? tmpdir) 
-	   (chdir tmpdir)
-	   (setup-build-directory (current-directory)) )
+    (cond ((file-exists? tmpdir) (chdir tmpdir))
 	  (else
 	   (create-directory tmpdir)
 	   (set! *tmpdir-created* #t)
 	   (chdir tmpdir)
 	   (setup-build-directory (current-directory))
-	   (let ((fn2 (string-append "../" filename))
-		 (v (setup-verbose-flag)) )
+	   (let ((fn2 (string-append "../" filename)))
 	     (if (testgz fn2)
-		 (run (gunzip -c ,fn2 |\|| tar ,(if v 'xvf 'xf) -))
-		 (run (tar ,(if v 'xvf 'xf) ,fn2)) ) ) ) )
+		 (run (gunzip -c ,fn2 |\|| tar xvf -))
+		 (run (tar xvf ,fn2)) ) ) ) )
     (set! *temporary-directory* tmpdir) ) )
 
 (define (copy-file from to)
   (let ((from (if (pair? from) (car from) from))
 	(to (if (pair? from) (make-pathname to (cadr from)) to)) )
     (ensure-directory to)
-    (run (,*copy-command* ,(quotewrap from) ,(quotewrap to)) ) ) )
+    (run (,*copy-command* ,from ,to)) ) )
 
 (define (move-file from to)
   (let ((from (if (pair? from) (car from) from))
 	(to (if (pair? from) (make-pathname to (cadr from)) to)) )
     (ensure-directory to)
-    (run (,*move-command* ,(quotewrap from) ,(quotewrap to)) ) ) )
+    (run (,*move-command* ,from ,to)) ) )
 
 (define (remove-file* dir)
-  (run (,*remove-command* ,(quotewrap dir)) ) )
+  (run (,*remove-command* ,dir)) )
 
 (define (make-dest-pathname path file)
   (match file
@@ -560,21 +526,14 @@ EOF
 	   (_ (error "invalid file-specification" f)) ) )
        flist) )
 
-(define (translate-extension f #!optional default)
-  (pathname-replace-extension f
-   (match (pathname-extension f)
-     (#f default)
-     ("so" ##sys#load-dynamic-extension)
-     ("o" (if *windows-shell* "obj" "o"))
-     ("a" (if *windows-shell* "lib" "a"))
-     (x x) ) ) )
-
 (define (install-extension id files #!optional (info '()))
-  (define (soify f) (translate-extension f))
+  (define (soify f)
+    (if (string=? (pathname-extension f) "so")
+	(pathname-replace-extension f ##sys#load-dynamic-extension)
+	f) )
   (when (setup-install-flag)
     (let* ((files (check-filelist (if (list? files) files (list files))))
 	   (rpath (repo-path))
-	   (rpathd (repo-path #t))
 	   (files (if *windows*
 		      (map (lambda (f) 
 			     (if (pair? f)
@@ -584,37 +543,22 @@ EOF
 		      files) ) 
 	   (dests (map (lambda (f)
 			 (let ((from (if (pair? f) (car f) f))
-			       (to (make-dest-pathname rpathd f)) )
-			   (when (and (not *windows*) (equal? "so" (pathname-extension to)))
+			       (to (make-dest-pathname rpath f)) )
+			   (when (and (not *windows*) (string=? "so" (pathname-extension to)))
 			     (run (,*remove-command* ,to)) )
-			   (copy-file from to)
-			   (unless *windows-shell*
-				   (run (chmod a+r ,to)))
-			   (and-let* ((static (assq 'static info)))
-			     (when (and (eq? (software-version) 'macosx)
-					(equal? (cadr static) from) )
-			       (run (ranlib ,to)) ) )
-			   (make-dest-pathname rpath f)))
+			   (copy-file from to) 
+			   to) )
 		       files) ) )
       (and-let* ((docs (assq 'documentation info)))
-	(let ((docpath (pathname-directory (doc-index #t))))
-	  (print "\n* Installing documentation files in " docpath ":")
-	  (for-each
-	   (lambda (f)
-	     (if (file-exists? f)
-		 (copy-file f (make-pathname docpath f))
-		 (print "Warning: file " f " doesn't seem to exist") )
-	     (print "  * " <>) (cdr docs)) 
-	   (newline)
-	   (set! *rebuild-doc-index* #t)) ) )
+	(print "\n* The following documentation files have been installed in " (doc-index) ":")
+	(for-each (cut print "  * " <>) (cdr docs)) 
+	(newline)
+	(set! *rebuild-doc-index* #t))
       (and-let* ((exs (assq 'examples info)))
-	(print "\n* Installing example files in " *example-directory* ":")
+	(print "\n* The following example files have been installed in " *example-directory* ":")
 	(for-each 
 	 (lambda (f)
 	   (copy-file f (make-pathname *example-directory* f))
-	   (unless *windows-shell*
-		   (run (chmod a+rx ,*example-directory*)))
-
 	   (print "  * " f) )
 	 (cdr exs))
 	(newline) )
@@ -622,12 +566,12 @@ EOF
 
 (define (install-program id files #!optional (info '()))
   (define (exify f)
-    (translate-extension
-     f
-     (if *windows-shell* "exe" #f) ) )
+    (if (not (pathname-extension f)) 
+	(pathname-replace-extension f "exe")
+	f) )
   (when (setup-install-flag)
     (let* ((files (check-filelist (if (list? files) files (list files))))
-	   (ppath (if *destdir* (make-pathname *destdir* "bin") (program-path)))
+	   (ppath (program-path)) 
 	   (files (if *windows*
                       (map (lambda (f)
                              (if (list? f) 
@@ -639,8 +583,6 @@ EOF
 			 (let ((from (if (pair? f) (car f) f))
 			       (to (make-dest-pathname ppath f)) )
 			   (copy-file from to) 
-			   (unless *windows-shell*
-				   (run (chmod a+r ,to)))
 			   to) )
 		       files) ) )
       (write-info id dests info) ) ) )
@@ -653,12 +595,9 @@ EOF
 			  (let ((from (if (pair? f) (car f) f))
 				(to (make-dest-pathname ppath f)) )
 			    (copy-file from to) 
-			    (unless *windows-shell*
-				    (run (chmod a+r ,to)))
-			    to) )
-			files) ) )
+			    to) ) ) ) )
       (unless *windows-shell*
-	(run (chmod a+rx ,(string-intersperse pfiles " "))) )
+	(run (chmod a+x ,(string-intersperse pfiles " "))) )
       (write-info id pfiles info) ) ) )
 
 (define (uninstall-extension ext)
@@ -671,18 +610,16 @@ EOF
 	   (lambda (f)
 	     (let ((f (if (pair? f) (cadr f) f)))
 	       (when (setup-verbose-flag) (printf "  deleting ~A~%" f))
-	       (run (,*remove-command* ,(quotewrap f)) ) ) )
+	       (run (,*remove-command* ,f)) ) )
 	   (cdr files) ) )
 	(print "no files to uninstall") )
     (when (assq 'documentation info) (set! *rebuild-doc-index* #t))
-    (delete-file* (make-setup-info-pathname (->string ext)))))
+    (let ((sfile (make-pathname (repository-path) (->string ext) setup-file-extension)))
+      (delete-file* sfile)
+      (delete-file* (pathname-replace-extension sfile alternative-setup-file-extension)) ) ) )
 
-(define (repo-path #!optional ddir?)
-  (let ((p (if (and ddir? *destdir*)
-	       (make-pathname 
-		(list *destdir* "lib/chicken") 
-		(pathname-file (repository-path))) ; we need the binary-compat. version
-	       (repository-path))) )
+(define (repo-path)
+  (let ((p (repository-path)))
     (ensure-directory p)
     p) )
 
@@ -691,65 +628,28 @@ EOF
     (if (file-exists? dir)
 	(unless (directory? dir)
 	  (error "can not create directory: a file with the same name already exists") )
-	(begin
-	  (create-directory dir)
-	  (unless *windows-shell*
-		  (run (chmod a+x ,dir)))))))
+	(create-directory dir) ) ) )
 
-(define (try-compile code #!key c++ (cc (if c++ *cxx* *cc*)) (cflags "") (ldflags "") (verb (setup-verbose-flag)) (compile-only #f))
+(define (test-compile code #!key (cflags "") (ldflags "") (verb (setup-verbose-flag)) (compile-only #f))
   (let* ((fname (create-temporary-file "c"))
 	 (oname (pathname-replace-extension fname "o"))
 	 (r (begin
 	      (with-output-to-file fname (cut display code))
 	      (system 
-	       (let ((cmd (conc
-			   cc " "
-			   (if compile-only "-c" "") " "
-			   cflags " " *target-cflags* " "
-			   fname " "
-			   (if compile-only
-			       "" 
-			       (conc ldflags " " *target-lflags*) )
-			   " "
-			   (if *msvc* "" ">/dev/null") " "
-			   (if verb "" "2>&1") ) ) )
+	       (let ((cmd (sprintf "~A ~A ~A ~A ~A ~A ~A"
+				   *cc*
+				   (if compile-only "-c" "")
+				   cflags
+				   fname
+				   (if compile-only "" ldflags) 
+				   (if *msvc* "" ">/dev/null")
+				   (if verb "" "2>&1") ) ) )
 		 (when verb (print cmd " ..."))
 		 cmd) ) ) ) )
     (when verb (print (if (zero? r) "succeeded." "failed.")))
-    (system (sprintf "~A ~A" *remove-command* (quotewrap fname)))
+    (system (sprintf "~A ~A" *remove-command* fname))
     (zero? r) ) )
 
-(define (required-chicken-version v)
-  (when (string-ci<? (chicken-version) (->string v))
-    (error (sprintf "CHICKEN version ~a or higher is required" v)) ) )
-
-(define (upgrade-message ext msg)
-  (error
-   (sprintf
-    "the required extension `~s' ~a - please run~%~%  chicken-setup ~a~%~%and repeat the current installation operation."
-    ext msg ext) ) )
-
-(define (required-extension-version . args)
-  (let loop ((args args))
-    (match args
-      (() #f)
-      ((ext version . more)
-       (let ((info (extension-information ext))
-	     (version (->string version)) )
-	 (if info
-	     (let ((ver (and (assq 'version info) (cadr (assq 'version info)))))
-	       (cond ((not ver) (upgrade-message ext "has no associated version information"))
-		     ((string-ci<? (->string ver) version)
-		      (upgrade-message 
-		       ext
-		       (sprintf "is older than ~a, which is what this extension requires"
-				version) ) )
-		     (else (loop more)) ) ) 
-	     (upgrade-message ext "is not installed") ) ) )
-      (_ (error 'required-extension-information "bad argument format" args)) ) ) )
-
-(define test-compile try-compile)
-
 (define (find-library name proc)
   (test-compile 
    (sprintf "#ifdef __cplusplus~%extern \"C\"~%#endif~%char ~a();~%int main() { ~a(); return 0; }~%" proc proc)
@@ -764,7 +664,7 @@ EOF
 
 (define (http-get-path-request path fname host)
   (sprintf "~A HTTP/1.0\r\nHost: ~A\r\nConnection: close\r\nContent-length: 0\r\n\r\n"
-	   (let ((p (make-pathname path fname "" "/")))
+	   (let ((p (make-pathname path fname)))
 	     (if (absolute-pathname? p)
 		 p
 		 (conc "/" p) ) )
@@ -782,8 +682,8 @@ EOF
 
 (define (download-repository-tree)
   (unless *repository-tree*
-    (when (setup-verbose-flag) (print "downloading catalog ..."))
-    (let loop ((hosts *repository-hosts*))
+    (print "downloading catalog ...")
+    (let loop ((hosts repository-hosts))
       (if (null? hosts)
 	  (error "unable to connect")
 	  (match hosts
@@ -792,8 +692,7 @@ EOF
 	      (lambda (return)
 		(or (handle-exceptions ex
 		      (begin (printf "could not connect to ~A.~%" host) #f)
-		      (when (setup-verbose-flag)
-			(printf "downloading catalog from ~A ...~%" host) )
+		      (printf "downloading catalog from ~A ...~%" host)
 		      (let-values (((i o) (setup-tcp-connect host port)))
 			(set! *last-decent-host* (car hosts))
 			(let ((req (http-get-request path remote-repository-name host)))
@@ -810,7 +709,6 @@ EOF
 			    (if (string=? "" ln)
 				(begin
 				  (set! *repository-tree* (read i))
-				  (set! *repository-tree-downloaded* #t)
 				  (when *debug*
 				    (print "catalog:")
 				    (pp *repository-tree*) )
@@ -818,8 +716,7 @@ EOF
 				  (close-output-port o)
 				  #t)
 				(loop) ) ) ) ) )
-		    (loop more) ) ) ) )
-	    ((x . _) (error "(internal) invalid host" x)) ) ) ) ) )
+		    (loop more) ) ) ) ) ) ) ) ) )
 
 (define *progress-indicator*
   (thread-start!
@@ -839,72 +736,36 @@ EOF
 	(thread-suspend! *progress-indicator*) ) ) )
 
 (define (download-data hostdata item #!optional filename)
-  (unless hostdata (set! hostdata (car *repository-hosts*)))
-  (cond (*local-repository*
-	 (when (setup-verbose-flag) (printf "fetching from local directory ~a ...~%" *local-repository*))
-	 (let ((p (->string item)))
-	   (copy-file (make-pathname *local-repository* p) (make-pathname #f p "egg-dir")) ) )
-	(*svn-repository*
-	 (when (setup-verbose-flag) (printf "fetching from svn repository ~a ...~%" *svn-repository*))
-	 (let ((p (->string item)))
-	   (run (svn co ,(if *revision* (conc "--revision " *revision*) "")
-		     ,(make-pathname *svn-repository* p) ,(make-pathname #f p "egg-dir"))) ) )
-	(else
-	 (match hostdata
-	   ((host path port)
-	    (let ((fname (or filename (third (assq item *repository-tree*)))))
-	      (printf "downloading ~A from ~A ~!" fname hostdata)
-	      (let-values (((i o) (setup-tcp-connect host port)))
-		(let ((req (http-get-request 
-			    (if filename (pathname-directory filename) path)
-			    (if filename (pathname-strip-directory fname) fname)
-			    host) ) )
-		  (when *debug* (display req))
-		  (display req o) )
-		(let loop ()
-		  (let ((ln (read-line i)))
-		    ;; check for 404 here...
-		    (if (string=? "" ln)
-			(let ((data (with-progress-indicator (cut read-string #f i))))
-			  (close-input-port i)
-			  (close-output-port o)
-			  (with-output-to-file (pathname-strip-directory fname)
-			    (cut display data) 
-			    binary:) ) 
-			(loop) ) ) ) ) ) )
-	   (x (error "(internal) invalid host" x)) ) ) ) )
+  (match hostdata
+    ((host path port)
+     (let ((fname (or filename (third (assq item *repository-tree*)))))
+       (printf "downloading ~A from ~A ~!" fname hostdata)
+       (let-values (((i o) (setup-tcp-connect host port)))
+	 (let ((req (http-get-request 
+		     (if filename (pathname-directory filename) path)
+		     (if filename (pathname-strip-directory fname) fname)
+		     host) ) )
+	   (when (setup-verbose-flag) (display req))
+	   (display req o) )
+	 (let loop ()
+	   (let ((ln (read-line i)))
+	     ;; check for 404 here...
+	     (if (string=? "" ln)
+		 (let ((data (with-progress-indicator (cut read-string #f i))))
+		   (close-input-port i)
+		   (close-output-port o)
+		   (with-output-to-file (pathname-strip-directory fname)
+		     (cut display data) binary:) ) 
+		 (loop) ) ) ) ) ) ) ) )
 
-(define (fetch-file ext)
-  (define (eval-req r)
-    (when (setup-verbose-flag) 
-      (print "Testing system:")
-      (pp r) )
-    (let ((f (eval r)))
-      (when (setup-verbose-flag) 
-	(print "\t-> " f) )
-      f) )
+(define (fetch-file-from-net ext)
   (define (requirements reqs)
     (fold 
      (lambda (r reqs)
-       (cond ((symbol? r)
-	      (let ((node (assq r *repository-tree*)))
-		(cond (node (append (requirements (cdddr node)) (list (car node)) reqs))
-		      ((memq r ##sys#core-library-modules) reqs)
-		      (else (error "Broken dependencies: extension does not exist" r) ) ) ) )
-	     ((and *repository-tree-downloaded*
-		   (not *windows*)
-		   (or (zero? (current-user-id))
-		       (not *dont-ask*) ) )
-	      (print "WARNING: executing system test retrieved through potentially insecure network:\n")
-	      (pp r) 
-	      (cond ((yes-or-no?
-		      "Do you want to execute this code ?"
-		      (if (zero? (current-user-id)) "no" "yes") )
-		     (requirements (eval-req r) ) )
-		    (else
-		     (print "Test cancelled - aborting")
-		     (abort-setup) ) ) )
-	     (else (requirements (eval-req r) )) ) ) 
+       (let ((node (assq r *repository-tree*)))
+	 (cond (node (append (list (car node)) (requirements (cdddr node)) reqs))
+	       ((memq r core-library-modules) reqs)
+	       (else (error "Broken dependencies: extension does not exist" r) ) ) ) ) 
      '() 
      reqs) )
   (and (or *dont-ask*
@@ -913,7 +774,7 @@ EOF
 	    "yes") )
        (cond ((pathname-directory ext)
 	      (printf "Warning: no repository index available, trying direct download...~%" ext)
-	      (set! *last-decent-host* (car *repository-hosts*))
+	      (set! *last-decent-host* (car repository-hosts))
 	      (set! *dont-ask* #t)
 	      (download-data
 	       *last-decent-host*
@@ -927,7 +788,7 @@ EOF
 		(when *debug* (printf "catalog entry: ~s~%" a))
 		(cond (a (let ((reqs (remove extension-info (delete-duplicates (requirements (cdddr a)) eq?))))
 			   (when (pair? reqs)
-			     (print "downloading required extensions " reqs " ...")
+			     (print "downloading required extensions ...")
 			     (for-each (cut download-data *last-decent-host* <>) reqs)
 			     (print "installing required extensions ...")
 			     (for-each (cut install <>) (map ->string reqs)) )
@@ -939,34 +800,17 @@ EOF
   (let ((df (not *fetch-only*)))
     (let loop ((filename filename))
       (cond ((and df (with-ext filename "setup")) => run-setup-script)
-	    ((or (with-ext filename "egg") (with-ext filename "egg-dir")) =>
+	    ((and df (with-ext filename "scm")) => simple-install)
+	    ((and df (with-ext filename "egg")) => 
 	     (lambda (f)
-	       (when df
-		 (unpack/enter f)
-		 (let ((sfile (pathname-replace-extension f "setup")))
-		   (when (and (not (file-exists? sfile)) (file-exists? "tags") )
-		     (let ((ds (sort (directory "tags") string>=?)))
-		       (when (pair? ds) 
-			 (let ((d (make-pathname "tags" (car ds))))
-			   (chdir d)
-			   (setup-build-directory d) ) ) ) )
-		   (loop sfile)
-		   (rmtmpdir) ) ) ) )
-	    ((fetch-file filename) 
+	       (unpack f)
+               (loop (pathname-replace-extension f "setup"))
+               (rmtmpdir) ) )
+	    ((fetch-file-from-net filename) 
 	     (when df (loop (pathname-file filename))) ) ) ) ) )
 
-(define (doc-index #!optional ddir?)
-  (make-pathname (repo-path ddir?) "index.html"))
-
-(define (extension-documented? rpath fn)
-  (let ([pn (make-setup-info-pathname fn rpath)])
-    (and (file-exists? pn)
-	 (with-input-from-file pn
-	   (lambda ()
-	     (not (not (alist-ref 'documentation (read) eq?))) ) ) ) ) )
-
-(define (delete-undocumented-extensions rpath lst)
-  (filter (cut extension-documented? rpath <>) lst) )
+(define (doc-index)
+  (make-pathname (repo-path) "index.html"))
 
 (define (build-doc-index)
   (let ((rpath (repository-path))
@@ -997,11 +841,9 @@ EOF
 		 (and-let* ((r (assq 'release info)))
 		   (printf " Release: ~a" (cadr r)))
 		 (display "</td></tr>\n") ) ) )
-	   (delete-undocumented-extensions 
-	    rpath
-	    (sort (delete-duplicates
-		   (grep "^[^.].*\\.*$" (map pathname-file (directory rpath))) string=?)
-		  string<?) ) )
+	   (sort (delete-duplicates
+		  (grep "^[^.].*\\.*$" (map pathname-file (directory rpath))) string=?)
+		 string<?) )
 	  (display "</table></center></body></font></html>\n") ) ) ) ) )
 
 (define (format-string str cols #!optional right (padc #\space))
@@ -1046,11 +888,11 @@ EOF
 
 (define (main args)
   (define (parse-host host eggdir)
-    (set! *repository-hosts*
+    (set! repository-hosts
       (cons (match (string-match "(.+)\\:([0-9]+)" host)
 	      ((_ host port) (list host (if eggdir "eggs" "") (string->number port)))
 	      (_ (list host (if eggdir "eggs" "") 80)) )
-	    *repository-hosts*) )  )
+	    repository-hosts) )  )
   (setup-root-directory *base-directory*)
   (let ((uinst #f)
 	(anydone #f))
@@ -1082,27 +924,16 @@ EOF
 	(("-repository" dir . more)
 	 (repository-path dir)
 	 (loop more) )
-	(("-tree" file . more)
-	 (set! *repository-tree* (with-input-from-file file read))
-	 (loop more) )
 	(("--" . more)
 	 (exit) )
 	(("-program-path")
 	 (print (program-path))
 	 (exit) )
-	(("-destdir" path . more)
-	 (set! *example-directory* (make-pathname path "examples"))
-	 (set! *destdir* path) 
-	 (installation-prefix path)
-	 (loop more) )
 	(("-program-path" dir . more)
 	 (program-path dir)
 	 (loop more) )
 	(("-version" . _)
-	 (printf "chicken-setup - ~A~%" (chicken-version #t))
-	 (exit) )
-	(("-release" . _)
-	 (print (chicken-version))
+	 (printf "chicken-setup - Version ~A, Build ~A~%" build-version build-number)
 	 (exit) )
 	(("-script" filename . args)
 	 (command-line-arguments args)
@@ -1117,7 +948,8 @@ EOF
 	 (loop more) )
 	(("-host" host . more)
 	 (match (string-match "http://(.*)" host)
-	   ((_ host) (parse-host host #t) )
+	   ((_ host)
+	    (parse-host host #t) )
 	   (_ (parse-host host #t)) )
 	 (loop more) )
 	(("-proxy" proxy . more)
@@ -1136,11 +968,6 @@ EOF
 	(("-csc-option" opt . more)
 	 (set! *csc-options* (append *csc-options* (list opt)))
 	 (loop more) )
-	(("-ls" ext . more)
-	 (and-let* ((info (extension-information ext))
-		    (files (assq 'files info)) )
-	   (for-each print (cdr files) ) )
-	 (exit) )
 	(("-dont-ask" . more)
 	 (set! *dont-ask* #t)
 	 (loop more) )
@@ -1156,32 +983,11 @@ EOF
 	(("-debug" . more)
 	 (set! *debug* #t)
 	 (loop more) )
-	(("-revision" rev . more)
-	 (set! *revision* rev)
-	 (loop more) )
 	(("-check" . more)
 	 (set! *check-repository* #t)
 	 (set! anydone #t)
 	 (loop more) )
-	(("-svn" url . more)
-	 (set! *svn-repository* url)
-	 (set! *dont-ask* #t)
-	 (loop more) )
-	(("-local" path . more)
-	 (set! *local-repository* path)
-	 (set! *dont-ask* #t)
-	 (loop more) )
-	(("-fetch-tree" . more)
-	 (set! *fetch-tree-only* #t)
-	 (set! anydone #t)
-	 (loop more) )
-	(("-test" . exts)
-	 (let ((missing (remove extension-info exts)))
-	   (cond ((pair? missing) 
-		  (print "the following extensions are currently not installed: " missing)
-		  1)
-		 (else (exit 0)) ) ) )
-	(((or "-run" "-script" "-proxy" "-host" "-csc-option" "-ls" "-destdir" "-tree" "-local" "-svn" "-eval"))
+	(((or "-run" "-script" "-proxy" "-host" "-csc-option"))
 	 (error "missing option argument" (car args)) )
 	((filename . more)
 	 (cond ((and (> (string-length filename) 0) (char=? #\- (string-ref filename 0)))
@@ -1198,7 +1004,6 @@ EOF
 		 (match (string-match "http://([^/]+)/(.+)" filename)
 		   ((_ host path)
 		    (parse-host host #f)
-		    (set! *dont-ask* #t)
 		    (conc "/" path) )
 		   (_ filename)) )
 		(loop more) ) ) )
@@ -1209,9 +1014,6 @@ EOF
 		 (printf "No setup scripts to process~%")
 		 (for-each (if uinst uninstall-extension install) setups) ) ) )
 	 (when *check-repository* (check-for-upgrades))
-	 (when *fetch-tree-only*
-	   (download-repository-tree)
-	   (pp *repository-tree*) )
 	 (when *rebuild-doc-index*
 	   (when (setup-verbose-flag) (printf "Rebuilding documentation index...\n"))
 	   (build-doc-index) )
@@ -1223,7 +1025,6 @@ EOF
       (exit -1) )
   (call/cc
    (lambda (return)
-     (set! *abort-hook* return)
-     (main (append (string-split (or (getenv "CHICKEN_SETUP_OPTIONS") ""))
-		   (command-line-arguments) ) ) ) )
+     (set! *abort-yes-or-no* return)
+     (main (command-line-arguments)) ) )
   (rmtmpdir) )
