@@ -5,7 +5,7 @@
 ;
 ;
 ;-----------------------------------------------------------------------------------------------------------
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -206,7 +206,7 @@
 ;   foldable -> <boolean>                    If true: variable names foldable standard-binding
 ;   boxed -> <boolean>                       If true: variable has to be boxed after closure-conversion
 ;   contractable -> <boolean>                If true: variable names contractable procedure
-;   inlinable -> <boolean>                  If true: variable names potentially inlinable procedure
+;   inlineable -> <boolean>                  If true: variable names potentially inlinable procedure
 ;   collapsable -> <boolean>                 If true: variable refers to collapsable constant
 ;   removable -> <boolean>                   If true: variable is not used
 ;   replacable -> <variable>                 Variable can be replaced by another variable
@@ -233,13 +233,20 @@
 
 (declare
  (unit compiler)
- (disable-warning var) )
-
-#>
-#ifndef C_USE_C_DEFAULTS
-# define C_INSTALL_SHARE_HOME NULL
+ (disable-warning var)
+ (foreign-declare #<<EOF
+#ifdef CMAKE_BUILD
+# include "chicken-paths.h"
+# include "stack-size.h"
+#elif defined(C_USE_C_DEFAULTS)
+# include "chicken-defaults.h"
+#else
+# define C_INSTALL_HOME NULL
 #endif
 
+#ifdef CMAKE_BUILD
+# include "stack-size.h"
+#endif
 #ifndef C_DEFAULT_TARGET_STACK_SIZE
 # define C_DEFAULT_TARGET_STACK_SIZE 0
 #endif
@@ -247,7 +254,8 @@
 #ifndef C_DEFAULT_TARGET_HEAP_SIZE
 # define C_DEFAULT_TARGET_HEAP_SIZE 0
 #endif
-<#
+EOF
+) )
 
 
 #{compiler
@@ -256,10 +264,10 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home decompose-lambda-list external-to-pointer defconstant-bindings constant-declarations
-  copy-node! error-is-extended-binding toplevel-scope toplevel-lambda-id
+  foreign-type-table-size copy-node! error-is-extended-binding toplevel-scope toplevel-lambda-id
   unit-name insert-timer-checks used-units external-variables require-imports-flag custom-declare-alist
   profile-info-vector-name finish-foreign-result pending-canonicalizations
-  foreign-declarations emit-trace-info block-compilation line-number-database-size
+  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
   always-bound-to-procedure block-globals make-block-variable-literal block-variable-literal? block-variable-literal-name
   target-heap-size target-stack-size valid-c-identifier?
   target-initial-heap-size internal-bindings source-filename dump-nodes source-info->string
@@ -267,7 +275,7 @@
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
   broken-constant-nodes inline-substitutions-enabled loop-lambda-names expand-profile-lambda
-  profile-lambda-list profile-lambda-index emit-profile expand-profile-lambda
+  profile-lambda-list profile-lambda-index emit-profile expand-profile-lambda no-c-syntax-checks
   direct-call-ids foreign-type-table first-analysis callback-names namespace-table disabled-warnings
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database! scan-toplevel-assignments
   compiler-warning import-table use-import-table
@@ -277,7 +285,7 @@
   expand-foreign-lambda* data-declarations emit-control-file-item expand-foreign-primitive
   process-declaration external-protos-first basic-literal? emit-line-info
   transform-direct-lambdas! expand-foreign-callback-lambda* debugging emit-unsafe-marker
-  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -285,14 +293,14 @@
   simple-lambda-node? compute-database-statistics print-program-statistics output gen gen-list 
   pprint-expressions-to-file foreign-type-check estimate-foreign-result-size scan-used-variables scan-free-variables
   topological-sort print-version print-usage initialize-analysis-database export-list csc-control-file
-  estimate-foreign-result-location-size compressed-literals-initializer
+  estimate-foreign-result-location-size compressed-literals-initializer parse-easy-ffi
   expand-foreign-callback-lambda default-optimization-passes default-optimization-passes-when-trying-harder
   units-used-by-default words-per-flonum disable-stack-overflow-checking
-  parameter-limit eq-inline-operator optimizable-rest-argument-operators postponed-initforms
+  parameter-limit eq-inline-operator optimizable-rest-argument-operators check-c-syntax postponed-initforms
   membership-test-operators membership-unfold-limit valid-compiler-options valid-compiler-options-with-argument
   make-random-name final-foreign-type real-name-table real-name set-real-name! safe-globals-flag
   location-pointer-map literal-compression-threshold compressed-literals compressable-literal
-  lookup-exports-file undefine-shadowed-macros process-lambda-documentation emit-syntax-trace-info
+  lookup-exports-file undefine-shadowed-macros
   generate-code make-variable-list make-argument-list generate-foreign-stubs foreign-type-declaration
   process-custom-declaration do-lambda-lifting file-requirements emit-closure-info export-file-name
   foreign-argument-conversion foreign-result-conversion foreign-type-convert-argument foreign-type-convert-result}
@@ -300,9 +308,8 @@
 (eval-when (compile eval)
   (match-error-control #:fail) )
 
-
 (include "tweaks")
-
+(include "parameters")
 
 (define-inline (gensym-f-id) (gensym 'f_))
 
@@ -312,7 +319,7 @@
   (define default-target-stack-size #f) )
 
 (eval-when (load)
-  (define-foreign-variable installation-home c-string "C_INSTALL_SHARE_HOME")
+  (define-foreign-variable installation-home c-string "C_INSTALL_HOME")
   (define-foreign-variable default-target-heap-size int "C_DEFAULT_TARGET_HEAP_SIZE")
   (define-foreign-variable default-target-stack-size int "C_DEFAULT_TARGET_STACK_SIZE") )
 
@@ -323,16 +330,6 @@
 (define user-pass-2 (make-parameter #f))
 (define user-post-analysis-pass (make-parameter #f))
 
-(define-constant foreign-type-table-size 301)
-(define-constant analysis-database-size 3001)
-(define-constant default-line-number-database-size 997)
-(define-constant inline-table-size 301)
-(define-constant constant-table-size 301)
-(define-constant real-name-table-size 997)
-(define-constant import-table-size 997)
-(define-constant default-literal-compression-threshold 50)
-(define-constant default-inline-max-size 10)
-
 
 ;;; Global variables containing compilation parameters:
 
@@ -368,6 +365,7 @@
 (define disable-stack-overflow-checking #f)
 (define namespace-table '())
 (define require-imports-flag #f)
+(define no-c-syntax-checks #f)
 (define emit-unsafe-marker #f)
 (define external-protos-first #f)
 (define do-lambda-lifting #f)
@@ -502,17 +500,16 @@
 			  t) ) ) ]
 		 [else x] ) )
 	  ((and (not-pair? x) (constant? x)) `(quote ,x))
-	  ((not-pair? x) (syntax-error "illegal atomic form" x))
+	  ((not-pair? x) (quit "syntax error - illegal atomic form `~s'" x))
 	  ((symbol? (car x))
 	   (let* ([head (car x)]
 		  [rest (cdr x)]
 		  [ln (get-line x)]
 		  [name (resolve head ae)] )
-	     (emit-syntax-trace-info x #f)
 	     (unless (proper-list? x)
 	       (if ln
-		   (syntax-error (sprintf "(in line ~s) - malformed expression" ln) x)
-		   (syntax-error "malformed expression" x)))
+		   (quit "syntax error in line ~s - malformed expression `~s'" ln x)
+		   (quit "syntax error - malformed expression `~s'" x) ) )
 	     (set! ##sys#syntax-error-culprit x)
 	     (let* ([x2 (cons name rest)]
 		    [xexpanded (##sys#macroexpand-1-local x2 me)] )
@@ -562,8 +559,7 @@
 				    (set! block-globals (cons var block-globals))
 				    var) ] ) ) )
 
-			((##core#undefined ##core#callunit ##core#primitive ##core#inline_ref 
-					   ##core#inline_loc_ref) x)
+			((##core#undefined ##core#callunit ##core#primitive ##core#inline_ref ##core#inline_loc_ref) x)
 
 			((##core#require-for-syntax)
 			 (let ([ids (map eval (cdr x))])
@@ -571,7 +567,15 @@
 			   (hash-table-update! 
 			    file-requirements 'syntax-requirements (cut lset-union eq? <> ids)
 			    (lambda () ids) )
-			   '(##core#undefined) ) )
+			   (let ([rs (##sys#lookup-runtime-requirements ids)])
+			     (hash-table-update!
+			      file-requirements 'runtime-requirements (cut lset-union eq? <> rs)
+			      (lambda () rs))
+			     (walk
+			      (if (null? rs)
+				  '(##core#undefined)
+				  `(##sys#require ,@(map (lambda (x) `',x) rs)) )
+			      ae me dest) ) ) )
 
 			((##core#require-extension)
 			 (walk
@@ -610,7 +614,7 @@
 			   `(let ,(map (lambda (alias b)
 					 (list alias (walk (cadr b) ae me (car b))) )
 				       aliases bindings)
-			      ,(walk (##sys#canonicalize-body (cddr x) (cut assq <> ae2) me dest)
+			      ,(walk (##sys#canonicalize-body (cddr x) (cut assq <> ae2) me)
 				     ae2
 				     me dest) ) ) )
 
@@ -629,17 +633,15 @@
 			    (lambda (vars argc rest)
 			      (let* ([aliases (map gensym vars)]
 				     [ae2 (append (map cons vars aliases) ae)]
-				     [body0 (##sys#canonicalize-body obody (cut assq <> ae2) me dest)]
-				     [body (walk body0 ae2 me #f)]
+				     [body 
+				      (walk 
+				       (##sys#canonicalize-body obody (cut assq <> ae2) me)
+				       ae2
+				       me #f) ]
 				     [llist2 
 				      (build-lambda-list
 				       aliases argc
 				       (and rest (list-ref aliases (posq rest vars))) ) ] )
-				(when dest
-				  (match body0
-				    (('begin (or (? string? doc) ('quote doc)) _ . more)
-				     (process-lambda-documentation dest doc) )
-				    (_ #f) ) )
 				(set-real-names! aliases vars)
 				(cond [(and dest emit-profile (eq? 'lambda name))
 				       (expand-profile-lambda dest llist2 body) ]
@@ -655,7 +657,7 @@
 				(ae2 (append (map cons vars aliases) ae))
 				[body 
 				 (walk 
-				  (##sys#canonicalize-body obody (cut assq <> ae2) me dest)
+				  (##sys#canonicalize-body obody (cut assq <> ae2) me)
 				  ae2
 				  me #f) ] )
 			   (set-real-names! aliases vars)
@@ -671,7 +673,7 @@
 			     (set! always-bound-to-procedure
 			       (lset-adjoin eq? always-bound-to-procedure var))
 			     (set! always-bound (lset-adjoin eq? always-bound var)) )
-			   (when (eq? var var0) ; global?
+			   (when (eq? var var0)
 			     (when (macro? var)
 			       (compiler-warning 
 				'var "assigned global variable `~S' is a macro ~A"
@@ -758,6 +760,7 @@
 				[name (if (pair? (cdddr x))
 					  (cadr (fourth x))
 					  (symbol->string var) ) ] )
+			   (check-c-syntax name 'define-foreign-variable)
 			   (set! foreign-variables
 			     (cons (list var type (if (string? name) name (symbol->string name)))
 				   foreign-variables))
@@ -776,9 +779,7 @@
 				    (walk
 				     `(begin
 					(##core#set! ,arg ,(first conv))
-					(##core#set! 
-					 ,ret 
-					 ,(if (pair? (cdr conv)) (second conv) '##sys#values)) ) 
+					(##core#set! ,ret ,(if (pair? (cdr conv)) (second conv) '##sys#values)) ) 
 				     ae me dest) ) ]
 				 [else
 				  (##sys#hash-table-set! foreign-type-table name type)
@@ -810,11 +811,7 @@
 			   `(let (,(let ([size (words (estimate-foreign-result-location-size type))])
 				     ;; Add 2 words: 1 for the header, 1 for double-alignment:
 				     ;; Note: C_a_i_bytevector takes number of words, not bytes
-				     (list 
-				      store
-				      `(##core#inline_allocate
-					("C_a_i_bytevector" ,(+ 2 size))
-					',size)) ) )
+				     (list store `(##core#inline_allocate ("C_a_i_bytevector" ,(+ 2 size)) ',size)) ) )
 			      ,(walk
 				`(begin
 				   ,@(if init
@@ -841,9 +838,7 @@
 			 (let* ([name (cadr (second x))]
 				[valexp (third x)]
 				[val (handle-exceptions ex
-					 ;; could show line number here
-					 (quit "error in constant evaluation of ~S for named constant ~S" 
-					       valexp name)
+					 (quit "error in constant evaluation of ~S for named constant ~S" valexp name)
 				       (if (collapsable-literal? valexp)
 					   valexp
 					   (eval `(let ,defconstant-bindings ,valexp)) ) ) ] )
@@ -882,12 +877,11 @@
 			     (let ([name (cadr name)])
 			       (if (valid-c-identifier? name)
 				   (set! callback-names (cons name callback-names))
-				   (quit "name `~S' of external definition is not a valid C identifier"
-					 name) ) )
+				   (quit "name `~S' of external definition is not a valid C identifier" name) ) )
 			     (when (or (not (proper-list? vars)) 
 				       (not (proper-list? atypes))
 				       (not (= (length vars) (length atypes))) )
-			       (syntax-error 
+			       (##sys#syntax-error-hook 
 				"non-matching or invalid argument list to foreign callback-wrapper"
 				vars atypes) )
 			     `(##core#foreign-callback-wrapper
@@ -912,8 +906,8 @@
 						   ((or '(const nonnull-c-string) 'nonnull-c-string)
 						    `((##sys#make-c-string (let () ,@(cddr lam)))))
 						   ((or '(const c-string*) 'c-string*)
-						    (syntax-error
-						     "`c-string*' is not a valid result type for callback procedures"
+						    (##sys#syntax-error-hook
+						     "`c-string*' is not a valid resulte type for callback procedures"
 						     name) )
 						   ((or 'c-string '(const c-string))
 						    `((let ((r (let () ,@(cddr lam))))
@@ -955,17 +949,15 @@
 				 [else (handle-call)] ) ) ) ) ] ) ) ) )
 
 	  ((not (proper-list? x))
-	   (syntax-error "malformed expression" x) )
+	   (quit "syntax error - malformed expression `~s'" x) )
 
 	  ((constant? (car x))
-	   (emit-syntax-trace-info x #f)
 	   (compiler-warning 'syntax "literal in operator position: ~S" x) 
 	   (mapwalk x ae me) )
 
 	  ((and (pair? (car x)) (eq? 'lambda (caar x)))
 	   (let ([lexp (car x)]
 		 [args (cdr x)] )
-	     (emit-syntax-trace-info x #f)
 	     (##sys#check-syntax 'lambda lexp '(lambda lambda-list . #(_ 1)))
 	     (let ([llist (cadr lexp)])
 	       (if (and (proper-list? llist) (= (length llist) (length args)))
@@ -976,15 +968,12 @@
 			 (,var ,@(cdr x)) )
 		      ae me dest) ) ) ) ) )
 	  
-	  (else
-	   (emit-syntax-trace-info x #f)
-	   (mapwalk x ae me)) ) )
+	  (else (mapwalk x ae me)) ) )
   
   (define (mapwalk xs ae me)
     (map (lambda (x) (walk x ae me #f)) xs) )
 
   (when (memq 'c debugging-chicken) (newline) (pretty-print exp))
-  (##sys#clear-trace-buffer)
   ;; Process visited definitions and main expression:
   (walk 
    `(begin
@@ -1001,11 +990,11 @@
   (define (check-decl spec minlen . maxlen)
     (let ([n (length (cdr spec))])
       (if (or (< n minlen) (> n (:optional maxlen 99999)))
-	  (syntax-error "invalid declaration" spec) ) ) )  
+	  (quit "syntax error in declaration: `~s'" spec) ) ) )  
   (call-with-current-continuation
    (lambda (return)
      (unless (pair? spec)
-       (syntax-error "invalid declaration specification" spec) )
+       (quit "invalid specification syntax: ~S" spec) )
      (case (car spec)
        ((uses)
 	(let ([us (cdr spec)])
@@ -1069,10 +1058,18 @@
 	(let ([fds (cdr spec)])
 	  (if (every string? fds)
 	      (set! foreign-declarations (append foreign-declarations fds))
-	      (syntax-error "invalid declaration" spec) ) ) )
+	      (quit "syntax error in declaration: `~S'" spec) )
+	  (check-c-syntax (apply string-append fds) 'foreign-declare) ) )
+       ((foreign-parse)
+	(let ([fds (cdr spec)])
+	  (if (every string? fds)
+	      (return 
+	       (##sys#compiler-toplevel-macroexpand-hook 
+		`(begin ,@(parse-easy-ffi (string-intersperse fds ""))) ) )
+	      (quit "syntax error in declaration: `~S'" spec) ) ) )
        ((custom-declare)
 	(if (or (not (list? spec)) (not (list? (cadr spec))) (< (length (cadr spec)) 3))
-	    (syntax-error "invalid declaration" spec)
+	    (quit "syntax error in declaration: `~S'" spec)
 	    (process-custom-declaration (cadr spec) (cddr spec)) ) )
        ((c-options)
 	(emit-control-file-item `(c-options ,@(cdr spec))) )
@@ -1084,6 +1081,7 @@
 	   `(post-process ,@(map (cut string-substitute "\\$@" file <>) (cdr spec))) ) ) )
        ((block) (set! block-compilation #t))
        ((separate) (set! block-compilation #f))
+       ((check-c-syntax) (set! no-c-syntax-checks #f))
        ((keep-shadowed-macros) (set! undefine-shadowed-macros #f))
        ((not)
 	(check-decl spec 1)
@@ -1112,6 +1110,7 @@
 	   (check-decl spec 1 1)
 	   (case (cadr spec)
 	     [(interrupts-enabled) (set! insert-timer-checks #f)]
+	     [(check-c-syntax) (set! no-c-syntax-checks #t)]
 	     [(safe) 
 	      (set! unsafe #t)
 	      (##match#set-error-control #:unspecified) ]
@@ -1133,9 +1132,9 @@
 	(when (and (list? spec) (= 3 (length spec)))
 	  (set! compressed-literals-initializer (third spec)) ) )
        ((emit-exports)
-	(cond ((null? (cdr spec))
-	       (quit "invalid `emit-exports' declaration" spec) )
-	      ((not export-file-name) (set! export-file-name (cadr spec))) ) )
+	(if (null? (cdr spec))
+	    (quit "invalid `emit-exports' declaration" spec)
+	    (set! export-file-name (cadr spec)) ) )
        ((emit-external-prototypes-first)
 	(set! external-protos-first #t) )
        ((lambda-lift) (set! do-lambda-lifting #t))
@@ -1242,6 +1241,7 @@
 	 [body (apply string-append (map cadr (cdddr exp)))]
  	 [argtypes (map car args)]
 	 [argnames (map cadr args)] )
+    (check-c-syntax body 'foreign-lambda*)
     (create-foreign-stub rtype #f argtypes argnames body #f #f) ) )
 
 (define (expand-foreign-callback-lambda* exp)
@@ -1250,6 +1250,7 @@
 	 [body (apply string-append (map cadr (cdddr exp)))]
  	 [argtypes (map car args)]
 	 [argnames (map cadr args)] )
+    (check-c-syntax body 'foreign-callback-lambda*)
     (create-foreign-stub rtype #f argtypes argnames body #t #t) ) )
 
 (define (expand-foreign-primitive exp)
@@ -1259,6 +1260,7 @@
 	 [body (apply string-append (map cadr (if hasrtype (cdddr exp) (cddr exp))))]
  	 [argtypes (map car args)]
 	 [argnames (map cadr args)] )
+    (check-c-syntax body 'foreign-primitive)
     (create-foreign-stub rtype #f argtypes argnames body #f #t) ) )
 
 
@@ -1655,7 +1657,7 @@
 
 	 ;; Make 'contractable, if it has a procedure as known value, has only one use and one call-site and
 	 ;;  if the lambda has no free non-global variables or is an internal lambda. Make 'inlinable if
-	 ;;  use/call count is not 1:
+	 ;;  use/call count is 1:
 	 (when value
 	   (let ((valparams (node-parameters value)))
 	     (when (and (eq? '##core#lambda (node-class value))
