@@ -1,6 +1,6 @@
 ;;;; easyffi.scm
 ;
-; Copyright (c) 2000-2005, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -34,12 +34,14 @@
 
 
 #{compiler debugging-chicken used-units register-ffi-macro parse-easy-ffi ffi-include-path-list 
-	   foreign-type-declaration foreign-type-table ffi-dont-include
+	   foreign-type-declaration foreign-type-table ffi-dont-include compiler-warning
 	   foreign-declarations explicit-use-flag check-c-syntax no-c-syntax-checks}
 
 (declare
   (unit easyffi)
+  (disable-warning var)
   (compress-literals)
+  (no-procedure-checks)
   (export parse-easy-ffi register-ffi-macro used-units foreign-declarations number-type
 	  ffi-include-path-list ffi-dont-include
 	  foreign-type-declaration explicit-use-flag debugging-chicken check-c-syntax
@@ -160,7 +162,7 @@
      (set! pp-process (and pp-process (not (assq (string->symbol name) macro-table))))
      (set! pp-conditional-stack (cons pp-process pp-conditional-stack)) ]
     [('pp-if . _)
-     (warning "preprocessor conditional `~A' ignored (assuming false)" c)
+     (compiler-warning 'ffi "preprocessor conditional `~A' ignored (assuming false)" c)
      (set! pp-process #f)
      (set! pp-conditional-stack (cons #f pp-conditional-stack)) ]
     [_ (when pp-process
@@ -280,6 +282,7 @@
     [`(unsigned long . ,more) (values 'unsigned-long more)]
     [`(u32 . ,more) (values 'unsigned-integer32 more)]
     [`(s32 . ,more) (values 'integer32 more)]
+    [`(s64 . ,more) (values 'integer64 more)]
     [`(void . ,more) (values 'void more)]
     [`(bool . ,more) (values 'bool more)]
     [`(symbol . ,more) (values 'symbol more)]
@@ -771,7 +774,7 @@
 
 (define (process-macro-def name type)
   (if (memq type '(* _))
-      (warning "can not compute macro type `~A' (ignored)" name)
+      (compiler-warning 'ffi "can not compute macro type `~A' (ignored)" name)
       (let* ([name2 (fix-name name)]
 	     [sname (->string name)] )
 	(emit `(define-foreign-variable ,name2 ,type ,sname))
@@ -830,7 +833,8 @@
 				       [('ref t) t]
 				       [_ (if io
 					      (begin
-						(warning 
+						(compiler-warning 
+						 'ffi
 						 "~A parameter used with non-pointer type"
 						 io arg)
 						arg)
@@ -1180,7 +1184,7 @@
       ['c-string '<string>]
       [(or 'unsigned-char 'int 'unsigned-int 'short 'unsigned-short 'unsigned-int32 'int32 'integer32)
        '<exact>]
-      [(or 'long 'unsigned-long 'integer32 'integer 'unsigned-integer 'unsigned-integer32) '<integer>]
+      [(or 'long 'unsigned-long 'integer32 'integer 'unsigned-integer 'unsigned-integer32 'integer64) '<integer>]
       [(or 'float 'double) '<inexact>]
       ['number '<number>]
       [('enum _) '<exact>]
