@@ -1,6 +1,6 @@
 ;;; extras.scm - Optional non-standard extensions
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -40,6 +40,31 @@
  (foreign-declare #<<EOF
 #define C_hashptr(x)   C_fix(x & C_MOST_POSITIVE_FIXNUM)
 #define C_mem_compare(to, from, n)   C_fix(C_memcmp(C_c_string(to), C_c_string(from), C_unfix(n)))
+
+static C_word fast_read_line_from_file(C_word str, C_word port, C_word size) {
+  int n = C_unfix(size);
+  int i;
+  char c;
+  char *buf = C_c_string(str);
+  C_FILEPTR fp = C_port_file(port);
+
+  if ((c = getc(fp)) == EOF)
+    return C_SCHEME_END_OF_FILE;
+
+  ungetc(c, fp);
+
+  for (i = 0; i < n; i++) {
+    c = getc(fp);
+    switch (c) {
+    case '\r':
+      if ((c = getc(fp)) != '\n') ungetc(c, fp);
+    case EOF: clearerr(fp);
+    case '\n': return C_fix(i);
+    }
+    buf[i] = c;
+  }
+  return C_SCHEME_FALSE;
+}
 EOF
 ) )
 
@@ -52,7 +77,7 @@ EOF
     (bound-to-procedure
      ##sys#check-char ##sys#check-exact ##sys#check-port ##sys#check-string ##sys#substring
      ##sys#for-each ##sys#map ##sys#setslot ##sys#allocate-vector ##sys#check-pair ##sys#not-a-proper-list-error 
-     ##sys#member ##sys#assoc ##sys#error ##sys#signal-hook ##sys#read-string!
+     ##sys#member ##sys#assoc ##sys#error ##sys#signal-hook
      ##sys#check-symbol ##sys#check-vector ##sys#floor ##sys#ceiling ##sys#truncate ##sys#round 
      ##sys#check-number ##sys#cons-flonum
      ##sys#flonum-fraction ##sys#make-port ##sys#fetch-and-check-port-arg ##sys#print ##sys#check-structure 
@@ -146,24 +171,12 @@ EOF
 (define (compose . fns)
   (define (rec f0 . fns)
     (if (null? fns)
-	f0
-	(lambda args
-	  (call-with-values
-	      (lambda () (apply (apply rec fns) args))
-	    f0) ) ) )
-  (if (null? fns)
-      values
-      (apply rec fns) ) )
-
-(define (o . fns)
-  (if (null? fns)
-      identity
-      (let loop ((fns fns))
-	(let ((h (##sys#slot fns 0))
-	      (t (##sys#slot fns 1)) )
-	  (if (null? t)
-	      h
-	      (lambda (x) (h ((loop t) x))))))))
+      f0
+      (lambda args
+        (call-with-values
+          (lambda () (apply (apply rec fns) args))
+          f0) ) ) )
+  (apply rec fns) )
 
 (define (list-of pred)
   (lambda (lst)
@@ -189,12 +202,10 @@ EOF
 		     (apply h args)
 		     (loop t) ) ) ) ) ) ) ) )
 
-(define (any? x) #t)
-
 
 ;;; List operators:
 
-(define (atom? x) (##core#inline "C_i_not_pair_p" x))
+(define (atom? x) (##core#inline "C_i_atomp" x))
 
 (define (tail? x y)
   (##sys#check-list y 'tail?)
@@ -225,6 +236,8 @@ EOF
 (define (flatten . lists0)
   (let loop ([lists lists0] [rest '()])
     (cond [(null? lists) rest]
+	  [(cond-expand [unsafe #f] [else (not (pair? lists))])
+	   (##sys#not-a-proper-list-error lists0) ]
 	  [else
 	   (let ([head (##sys#slot lists 0)]
 		 [tail (##sys#slot lists 1)] )
@@ -367,108 +380,89 @@ EOF
     (lambda args
       (let* ([parg (pair? args)]
 	     [p (if parg (car args) ##sys#standard-input)]
-	     [limit (and parg (pair? (cdr args)) (cadr args))])
+	     [limit (and parg (pair? (cdr args)) (cadr args))]
+	     [buffer-len (if limit limit 256)]
+	     [buffer (make-string buffer-len)])
 	(##sys#check-port p 'read-line)
-	(cond ((##sys#slot p 8) => (lambda (rl) (rl p limit)))
-	      (else
-	       (let* ((buffer-len (if limit limit 256))
-		      (buffer (##sys#make-string buffer-len)))
-		 (let loop ([i 0])
-		   (if (and limit (fx>= i limit))
-		       (##sys#substring buffer 0 i)
-		       (let ([c (##sys#read-char-0 p)])
-			 (if (eof-object? c)
-			     (if (fx= i 0)
-				 c
-				 (##sys#substring buffer 0 i) ) 
-			     (case c
-			       [(#\newline) (##sys#substring buffer 0 i)]
-			       [(#\return)
-				(let ([c (peek-char p)])
-				  (if (char=? c #\newline)
-				      (begin (##sys#read-char-0 p)
-					     (##sys#substring buffer 0 i))
-				      (##sys#substring buffer 0 i) ) ) ]
-			       [else
-				(when (fx>= i buffer-len)
-				  (set! buffer (##sys#string-append buffer (make-string buffer-len)))
-				  (set! buffer-len (fx+ buffer-len buffer-len)) )
-				(##core#inline "C_setsubchar" buffer i c)
-				(loop (fx+ i 1)) ] ) ) ) ) ) ) ) ) ) ) ) )
+	(if (eq? 'stream (##sys#slot p 7))
+	    (begin
+	      (let loop ([len buffer-len]
+			 [buffer buffer]
+			 [result ""]
+			 [f #f])
+		(let ([n (##core#inline "fast_read_line_from_file" buffer p len)])
+		  (cond [(eof-object? n) (if f result #!eof)]
+			[(and limit (not n))
+			 (##sys#string-append result (##sys#substring buffer 0 limit))]
+			[(not n)
+			 (loop (fx* len 2) (make-string (fx* len 2))
+			       (##sys#string-append result
+						    (##sys#substring buffer 0 (fx- len 1)))
+			       #t) ]
+			[f (##sys#setslot p 4 (fx+ (##sys#slot p 4) 1))
+			   (##sys#string-append result (##sys#substring buffer 0 n))]
+			[else
+			 (##sys#setslot p 4 (fx+ (##sys#slot p 4) 1))
+			 (##sys#substring buffer 0 n)] ) ) ))
+	    (let loop ([i 0])
+	      (if (and limit (fx>= i limit))
+		  (##sys#substring buffer 0 i)
+		  (let ([c (##sys#read-char-0 p)])
+		    (if (eof-object? c)
+			(if (fx= i 0)
+			    c
+			    (##sys#substring buffer 0 i) ) 
+			(case c
+			  [(#\newline) (##sys#substring buffer 0 i)]
+			  [(#\return)
+			   (let ([c (peek-char p)])
+			     (if (char=? c #\newline)
+				 (begin (##sys#read-char-0 p)
+					(##sys#substring buffer 0 i))
+				 (##sys#substring buffer 0 i) ) ) ]
+			  [else
+			   (when (fx>= i buffer-len)
+				 (set! buffer (##sys#string-append buffer (make-string buffer-len)))
+				 (set! buffer-len (fx+ buffer-len buffer-len)) )
+			   (##core#inline "C_setsubchar" buffer i c)
+			   (loop (fx+ i 1)) ] ) ) ) ) ) ) ) ) ) )
 
 (define read-lines
-  (let ((read-line read-line)
-	(call-with-input-file call-with-input-file) 
-	(reverse reverse) )
+  (let ([read-line read-line]
+	[call-with-input-file call-with-input-file] 
+	[reverse reverse] )
     (lambda port-and-max
-      (let* ((port (if (pair? port-and-max) (##sys#slot port-and-max 0) ##sys#standard-input))
-	     (rest (and (pair? port-and-max) (##sys#slot port-and-max 1)))
-	     (max (if (pair? rest) (##sys#slot rest 0) #f)) )
+      (let* ([port (if (pair? port-and-max) (##sys#slot port-and-max 0) ##sys#standard-input)]
+	     [rest (and (pair? port-and-max) (##sys#slot port-and-max 1))]
+	     [max (if (pair? rest) (##sys#slot rest 0) #f)] )
 	(define (doread port)
-	  (let loop ((lns '())
-		     (n (or max 1000000)) )
-	    (if (eq? n 0)
-		(reverse lns)
-		(let ((ln (read-line port)))
-		  (if (eof-object? ln)
-		      (reverse lns)
-		      (loop (cons ln lns) (fx- n 1)) ) ) ) ) )
+	  (do ([ln (read-line port) (read-line port)]
+	       [lns '() (cons ln lns)]
+	       [n (or max 1000000) (fx- n 1)] )
+	      ((or (eof-object? ln) (eq? n 0)) (reverse lns)) ) )
 	(if (string? port)
 	    (call-with-input-file port doread)
 	    (begin
 	      (##sys#check-port port 'read-lines)
 	      (doread port) ) ) ) ) ) )
 
-(define (##sys#read-string! n dest port start)
-  (when (##sys#slot port 6)		; peeked?
-    (##core#inline "C_setsubchar" dest start (##sys#read-char-0 port))
-    (set! start (fx+ start 1)) )
-  (let ((rdstring (##sys#slot (##sys#slot port 2) 7)))
-    (let loop ((start start) (n n) (m 0))
-      (let ((n2 (if rdstring
-		    (rdstring port n dest start) ; *** doesn't update port-position!
-		    (let ((c (##sys#read-char-0 port)))
-		      (if (eof-object? c)
-			  0
-			  (begin
-			    (##core#inline "C_setsubchar" dest start c)
-			    1) ) ) ) ) )
-	(cond ((eq? n2 0) m)
-	      ((or (not n) (fx< n2 n)) 
-	       (loop (fx+ start n2) (and n (fx- n n2)) (fx+ m n2)) )
-	      (else (fx+ n2 m))) ) ) ))
-
-(define (read-string! n dest #!optional (port ##sys#standard-input) (start 0))
-  (##sys#check-port port 'read-string!)
-  (##sys#check-string dest 'read-string!)
-  (when n
-    (##sys#check-exact n 'read-string!)
-    (when (fx> (fx+ start n) (##sys#size dest))
-      (set! n (fx- (##sys#size dest) start))))
-  (##sys#check-exact start 'read-string!)
-  (##sys#read-string! n dest port start) )
-
 (define read-string
-  (let ((open-output-string open-output-string)
-	(get-output-string get-output-string) )
-    (lambda (#!optional n (p ##sys#standard-input))
-      (##sys#check-port p 'read-string)
-      (cond (n (##sys#check-exact n 'read-string)
-	       (let* ((str (##sys#make-string n))
-		      (n2 (##sys#read-string! n str p 0)) )
-		 (if (eq? n n2)
-		     str
-		     (##sys#substring str 0 n2))))
-	    (else
-	     (let ([str (open-output-string)])
-	       (let loop ([n n])
-		 (or (and (eq? n 0) (get-output-string str))
-		     (let ([c (##sys#read-char-0 p)])
-		       (if (eof-object? c)
-			   (get-output-string str)
-			   (begin
-			     (##sys#write-char c str) 
-			     (loop (and n (fx- n 1))) ) ) ) ) ) ) ) ) ) ) )
+  (let ([open-output-string open-output-string]
+	[get-output-string get-output-string] )
+    (lambda n-and-port
+      (let-optionals n-and-port ([n #f] [p ##sys#standard-input])
+	(##sys#check-port p 'read-string)
+	(let ([str (open-output-string)])
+	  (when n (##sys#check-exact n 'read-string))
+	  (##sys#check-port p 'read-string)
+	  (let loop ([n n])
+	    (or (and (eq? n 0) (get-output-string str))
+		(let ([c (##sys#read-char-0 p)])
+		  (if (eof-object? c)
+		      (get-output-string str)
+		      (begin
+			(##sys#write-char c str) 
+			(loop (and n (fx- n 1))) ) ) ) ) ) ) ) ) ) )
 
 (define read-token
   (let ([open-output-string open-output-string]
@@ -567,8 +561,9 @@ EOF
 ;   10: last
 
 (define make-input-port
-  (lambda (read ready? close #!optional peek read-string read-line)
-    (let* ((class
+  (lambda (read ready? close . peek)
+    (let* ([peek (and (pair? peek) (car peek))]
+	   [class
 	    (vector 
 	     (lambda (p)		; read-char
 	       (let ([last (##sys#slot p 10)])
@@ -592,19 +587,17 @@ EOF
 	       (##sys#setislot p 8 #t) )
 	     #f				; flush-output
 	     (lambda (p)		; char-ready?
-	       (ready?) )
-	     read-string 		; read-string
-	     read-line) )		; read-line
-	   (data (vector #f))
-	   (port (##sys#make-port #t class "(custom)" 'custom)) )
+	       (ready?) ) ) ] 
+	   [data (vector #f)] 
+	   [port (##sys#make-port #t class "(custom)" 'custom)] )
       (##sys#setslot port 9 data) 
       port) ) )
 
 (define make-output-port
   (let ([string string])
     (lambda (write close . flush)
-      (let* ((flush (and (pair? flush) (car flush)))
-	     (class
+      (let* ([flush (and (pair? flush) (car flush))]
+	     [class
 	      (vector
 	       #f			; read-char
 	       #f			; peek-char
@@ -617,11 +610,9 @@ EOF
 		 (##sys#setislot p 8 #t) )
 	       (lambda (p)		; flush-output
 		 (when flush (flush)) )
-	       #f			; char-ready?
-	       #f			; read-string
-	       #f) )			; read-line
-	     (data (vector #f))
-	     (port (##sys#make-port #f class "(custom)" 'custom)) )
+	       #f) ]			; char-ready?
+	     [data (vector #f)] 
+	     [port (##sys#make-port #f class "(custom)" 'custom)] )
 	(##sys#setslot port 9 data) 
 	port) ) ) )
 
@@ -635,6 +626,7 @@ EOF
 ; Modified by felix for use with CHICKEN
 ;
 
+
 (define generic-write
   (let ([open-output-string open-output-string]
 	[get-output-string get-output-string] )
@@ -1227,46 +1219,47 @@ EOF
 	(##sys#check-port port 'fprintf)
 	(let ((index 0)
 	      (len (##sys#size msg)) )
+
 	  (define (fetch)
 	    (let ((c (##core#inline "C_subchar" msg index)))
-	      (set! index (fx+ index 1))
+	      (set! index (##core#inline "C_fixnum_plus" index 1))
 	      c) )
+
 	  (define (next)
 	    (if (cond-expand [unsafe #f] [else (##core#inline "C_eqp" args '())])
 		(##sys#error 'fprintf "too few arguments to formatted output procedure")
 		(let ((x (##sys#slot args 0)))
 		  (set! args (##sys#slot args 1)) 
 		  x) ) )
-	  (let loop ()
-	    (unless (fx>= index len)
-	      (let ((c (fetch)))
-		(if (and (eq? c #\~) (fx< index len))
-		    (let ((dchar (fetch)))
-		      (case (char-upcase dchar)
-			((#\S) (write (next) port))
-			((#\A) (display (next) port))
-			((#\C) (##sys#write-char-0 (next) port))
-			((#\B) (display (##sys#number->string (next) 2) port))
-			((#\O) (display (##sys#number->string (next) 8) port))
-			((#\X) (display (##sys#number->string (next) 16) port))
-			((#\!) (##sys#flush-output port))
-			((#\?)
-			 (let* ([fstr (next)]
-				[lst (next)] )
-			   (##sys#check-list lst 'fprintf)
-			   (rec fstr lst) ) )
-			((#\~) (##sys#write-char-0 #\~ port))
-			((#\%) (newline port))
-			((#\% #\N) (newline port))
-			(else
-			 (if (char-whitespace? dchar)
-			     (let skip ((c (fetch)))
-			       (if (char-whitespace? c)
-				   (skip (fetch))
-				   (set! index (fx- index 1)) ) )
-			     (##sys#error 'fprintf "illegal format-string character" dchar) ) ) ) )
-		    (##sys#write-char-0 c port) )
-		(loop) ) ) ) ) ) ) ) )
+
+	  (do ([c (fetch) (fetch)])
+	      ((fx> index len))
+	    (if (eq? c #\~)
+		(let ((dchar (fetch)))
+		  (case (char-upcase dchar)
+		    ((#\S) (write (next) port))
+		    ((#\A) (display (next) port))
+		    ((#\C) (##sys#write-char-0 (next) port))
+		    ((#\B) (display (##sys#number->string (next) 2) port))
+		    ((#\O) (display (##sys#number->string (next) 8) port))
+		    ((#\X) (display (##sys#number->string (next) 16) port))
+		    ((#\!) (##sys#flush-output port))
+		    ((#\?)
+		     (let* ([fstr (next)]
+			    [lst (next)] )
+		       (##sys#check-list lst 'fprintf)
+		       (rec fstr lst) ) )
+		    ((#\~) (##sys#write-char-0 #\~ port))
+		    ((#\%) (newline port))
+		    ((#\% #\N) (newline port))
+		    (else
+		     (if (char-whitespace? dchar)
+			 (let skip ((c (fetch)))
+			   (if (char-whitespace? c)
+			       (skip (fetch))
+			       (set! index (##core#inline "C_fixnum_difference" index 1)) ) )
+			 (##sys#error 'fprintf "illegal format-string character" dchar) ) ) ) )
+		(##sys#write-char-0 c port) ) ) ) ) ) ) )
 
 
 (define printf
@@ -1453,7 +1446,7 @@ EOF
 	(and (fx> len 0)
 	     (let loop ([ps 0]
 			[pe len] )
-	       (let ([p (fx+ ps (##core#inline "C_fixnum_divide" (fx- pe ps) 2))])
+	       (let ([p (fx+ ps (fx/ (fx- pe ps) 2))])
 		 (let* ([x (##sys#slot vec p)]
 			[r (proc x)] )
 		   (cond [(fx= r 0) p]
@@ -1564,7 +1557,7 @@ EOF
 			   (fx+ i 1)
 			   (fx- len 1) ) ) ) ) ) ) )
     (##sys#check-exact limit 'hash)
-    (##core#inline "C_fixnum_modulo" (fxand (foreign-value "C_MOST_POSITIVE_FIXNUM" int) (rechash x 0)) limit) ) )
+    (##core#inline "C_fixnum_modulo" (fxand #x00ffffff (rechash x 0)) limit) ) )
 
 (define (hash x #!optional (bound default-hash-bound))
   (##sys#check-exact bound 'hash)
@@ -1575,14 +1568,14 @@ EOF
 (define (string-hash s #!optional (bound default-hash-bound))
   (##core#inline 
    "C_fixnum_modulo"
-   (##core#inline "C_hash_string" s)
-   bound) )
+   (fxand #x00ffffff (##core#inline "C_hash_string" s))
+   limit) )
 
 (define (string-ci-hash s #!optional (bound default-hash-bound))
   (##core#inline 
    "C_fixnum_modulo"
-   (##core#inline "C_hash_string_ci" s)
-   bound) )
+   (fxand #x00ffffff (##core#inline "C_hash_string_ci" s))
+   limit) )
 
 
 ;;; Access:
@@ -1592,7 +1585,7 @@ EOF
   (##sys#slot ht 2) )
 
 (define hash-table-update! 
-  ;; This one was suggested by Sven Hartrumpf (and subsequently added in SRFI-69)
+  ;; This one was suggested by Sven Hartrumpf.
   (let ([eq0 eq?]
 	[floor floor] )
     (lambda (ht key proc #!optional
@@ -1620,29 +1613,21 @@ EOF
 		    ;; Fast path (eq? test):
 		    (let loop ((bucket bucket0))
 		      (cond ((eq? bucket '())
-			     (let ((val (proc (init))))
-			       (##sys#setslot vec k (cons (cons key val) bucket0))
-			       (##sys#setslot ht 2 c) 
-			       val) )
+			     (##sys#setslot vec k (cons (cons key (proc (init))) bucket0))
+			     (##sys#setslot ht 2 c) )
 			    (else
 			     (let ((b (##sys#slot bucket 0)))
 			       (if (eq? key (##sys#slot b 0))
-				   (let ((val (proc (##sys#slot b 1))))
-				     (##sys#setslot b 1 val)
-				     val)
+				   (##sys#setslot b 1 (proc (##sys#slot b 1)))
 				   (loop (##sys#slot bucket 1)) ) ) ) ) )
 		    (let loop ((bucket bucket0))
 		      (cond ((eq? bucket '())
-			     (let ((val (proc (init))))
-			       (##sys#setslot vec k (cons (cons key val) bucket0))
-			       (##sys#setslot ht 2 c) 
-			       val) )
+			     (##sys#setslot vec k (cons (cons key (proc (init))) bucket0))
+			     (##sys#setslot ht 2 c) )
 			    (else
 			     (let ((b (##sys#slot bucket 0)))
 			       (if (test key (##sys#slot b 0))
-				   (let ((val (proc (##sys#slot b 1))))
-				     (##sys#setslot b 1 val)
-				     val) 
+				   (##sys#setslot b 1 (proc (##sys#slot b 1)))
 				   (loop (##sys#slot bucket 1)) ) ) ) ) ) ) ) ) ) ) ) ) )
 
 (define hash-table-update!/default 
