@@ -1,6 +1,6 @@
 ;;;; optimizer.scm - The CHICKEN Scheme compiler (optimizations)
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -41,10 +41,10 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home decompose-lambda-list external-to-pointer
-  copy-node! export-list inline-list not-inline-list
+  foreign-type-table-size copy-node! export-list inline-list not-inline-list
   unit-name insert-timer-checks used-units external-variables
   debug-info-index debug-info-vector-name profile-info-vector-name
-  foreign-declarations emit-trace-info block-compilation line-number-database-size
+  foreign-declarations emit-trace-info block-compilation analysis-database-size line-number-database-size
   always-bound-to-procedure block-globals make-block-variable-literal block-variable-literal? block-variable-literal-name
   target-heap-size target-stack-size constant-declarations
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
@@ -58,7 +58,7 @@
   reorganize-recursive-bindings substitution-table simplify-named-call compiler-warning
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! expand-foreign-callback-lambda* debug-lambda-list debug-variable-list debugging
-  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
   string->c-identifier c-ify-string words check-and-open-input-file close-checked-input-file fold-inner constant?
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
   put! collect! count! get-line get-line-2 find-lambda-container display-analysis-database varnode qnode 
@@ -79,8 +79,7 @@
   (match-error-control #:fail) )
 
 (include "tweaks")
-
-(define-constant maximal-number-of-free-variables-for-liftable 16)
+(include "parameters")
 
 
 ;;; Scan toplevel expressions for assignments:
@@ -894,7 +893,7 @@
     ((3) ; classargs = (<var>)
      (and inline-substitutions-enabled
 	  (null? callargs)
-	  (or (test name 'standard-binding) (test name 'extended-binding))
+	  (test name 'standard-binding)
 	  (make-node '##core#call '(#t) (list cont (varnode (first classargs)))) ) )
 
     ;; (<op> a b) -> (<primitiveop> a (quote <i>) b)
