@@ -1,6 +1,6 @@
 ;;;; parameters.scm - Miscellaneous system parameters
 ;
-; Copyright (c) 2000-2004, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -33,7 +33,7 @@
 ; Germany
 
 
-(define-constant copyright "(c)2000-2005 Felix L. Winkelmann")
+(define-constant copyright "(c)2000-2006 Felix L. Winkelmann")
 
 (define-constant namespace-size 997)
 (define-constant namespace-max-id-len 31)
@@ -69,15 +69,14 @@
 (define-constant inline-table-size 301)
 (define-constant constant-table-size 301)
 (define-constant real-name-table-size 997)
+(define-constant import-table-size 997)
 
 (define-constant default-literal-compression-threshold 50)
 
 (define-constant maximal-number-of-free-variables-for-liftable 16)
 
-(define-constant unix-default-dynamic-load-libraries '("libchicken"))
+(define-constant default-dynamic-load-libraries '("libchicken"))
 (define-constant cygwin-default-dynamic-load-libraries '("cygchicken-0"))
-(define-constant mingw-default-dynamic-load-libraries '("libchicken-0"))
-(define-constant msvc-default-dynamic-load-libraries '("libchicken"))
 
 (define-constant macosx-load-library-extension ".dylib")
 (define-constant windows-load-library-extension ".dll")
@@ -92,6 +91,7 @@
 
 (define-constant repository-environment-variable "CHICKEN_REPOSITORY")
 (define-constant home-environment-variable "CHICKEN_HOME")
+(define-constant prefix-environment-variable "CHICKEN_PREFIX")
 
 (define-constant default-profile-name "PROFILE")
 
@@ -100,18 +100,15 @@
 (define-constant core-library-modules
   '(extras lolevel tinyclos utils tcp regex posix match srfi-1 srfi-4 srfi-14 srfi-18 srfi-13))
 
-(define-constant specially-handled-syntax-files
-  '((match match-support)) )
-
 (define-constant special-syntax-files
-  '(chicken-match-macros chicken-ffi-macros chicken-more-macros) )
+  '(chicken-ffi-macros chicken-more-macros) )
 
 (define-constant builtin-features
-  '(chicken srfi-23 srfi-30 srfi-39 srfi-6 srfi-10 
+  '(chicken srfi-23 srfi-30 srfi-39 srfi-6 srfi-10 srfi-2 srfi-31
 	    srfi-69 srfi-28) )		; these are actually in extras, but that is used by default
 
 (define-constant builtin-features/compiled
-  '(srfi-11 srfi-8 srfi-6 srfi-16 srfi-15 srfi-26 srfi-55 srfi-9 srfi-17) )
+  '(easyffi srfi-11 srfi-8 srfi-6 srfi-16 srfi-15 srfi-26 srfi-55 srfi-9 srfi-17) )
 
 (define-constant installed-executables
   '("chicken" "csc" "csi" "chicken-setup" "chicken-profile") )
