@@ -1,6 +1,6 @@
 ;;;; profiler.scm - Support code for profiling applications
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -49,7 +49,7 @@
     (no-bound-checks) ) ] )
 
 
-(define-constant profile-info-entry-size 5)
+(include "parameters.scm")
 
 
 ;;; Globals:
@@ -81,9 +81,6 @@
 	(set! ##sys#profile-vector-list (cons vec ##sys#profile-vector-list))
 	vec) ) ) )
 
-(define (##sys#set-profile-info-vector! vec i x)
-  (##sys#setslot vec (* i profile-info-entry-size) x) )
-
 
 ;;; Entry and exit into/out of profiled lambda:
 
