@@ -1,6 +1,6 @@
 ;;;; support.scm - Miscellaneous support code for the CHICKEN compiler
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -42,23 +42,23 @@
   non-foldable-standard-bindings foldable-standard-bindings non-foldable-extended-bindings foldable-extended-bindings
   standard-bindings-that-never-return-false side-effect-free-standard-bindings-that-never-return-false
   installation-home optimization-iterations compiler-cleanup-hook decompose-lambda-list
-  file-io-only banner custom-declare-alist disabled-warnings
+  foreign-type-table-size file-io-only banner custom-declare-alist parse-easy-ffi disabled-warnings
   unit-name insert-timer-checks used-units source-filename pending-canonicalizations
-  foreign-declarations block-compilation line-number-database-size
+  foreign-declarations block-compilation analysis-database-size line-number-database-size
   target-heap-size target-stack-size check-global-exports check-global-imports
   default-default-target-heap-size default-default-target-stack-size verbose-mode original-program-size
   current-program-size line-number-database-2 foreign-lambda-stubs immutable-constants foreign-variables
   rest-parameters-promoted-to-vector inline-table inline-table-used constant-table constants-used mutable-constants
-  dependency-list broken-constant-nodes inline-substitutions-enabled emit-syntax-trace-info
+  dependency-list broken-constant-nodes inline-substitutions-enabled no-c-syntax-checks
   always-bound-to-procedure block-variable-literal copy-node! valid-c-identifier? tree-copy copy-node-tree-and-rename
-  direct-call-ids foreign-type-table first-analysis scan-sharp-greater-string
+  direct-call-ids foreign-type-table first-analysis scan-sharp-greater-string enable-sharp-greater-read-syntax
   expand-profile-lambda profile-lambda-list profile-lambda-index profile-info-vector-name
   initialize-compiler canonicalize-expression expand-foreign-lambda update-line-number-database scan-toplevel-assignments
   perform-cps-conversion analyze-expression simplifications perform-high-level-optimizations perform-pre-optimization!
-  reorganize-recursive-bindings substitution-table simplify-named-call
+  reorganize-recursive-bindings substitution-table simplify-named-call check-c-syntax
   perform-closure-conversion prepare-for-code-generation compiler-source-file create-foreign-stub expand-foreign-lambda*
   transform-direct-lambdas! finish-foreign-result compressable-literal csc-control-file
-  debugging-chicken bomb check-signature posq stringify symbolify build-lambda-list
+  debugging-chicken bomb check-signature posq stringify symbolify flonum? build-lambda-list
   string->c-identifier c-ify-string words words->bytes check-and-open-input-file close-checked-input-file fold-inner
   constant? basic-literal? source-info->string import-table
   collapsable-literal? immediate? canonicalize-begin-body extract-mutable-constants string->expr get get-all
@@ -74,10 +74,11 @@
   default-optimization-iterations chop-separator chop-extension follow-without-loop dump-exported-globals
   generate-code make-variable-list make-argument-list generate-foreign-stubs foreign-type-declaration
   foreign-argument-conversion foreign-result-conversion final-foreign-type debugging export-list block-globals
-  lookup-exports-file constant-declarations process-lambda-documentation
+  lookup-exports-file constant-declarations
   make-random-name foreign-type-convert-result foreign-type-convert-argument process-custom-declaration}
 
 
+(include "parameters")
 (include "tweaks")
 (include "banner")
 
@@ -121,16 +122,10 @@
 (set! ##sys#syntax-error-hook
   (lambda (msg . args)
     (let ([out (current-error-port)])
-      (fprintf out "Syntax error: ~a~%~%" msg) 
-      (for-each (cut fprintf out "\t~s~%" <>) args)
-      (print-call-chain out 0 ##sys#current-thread "\n\tExpansion history:\n")
+      (fprintf out "Syntax error: ~a~%" msg) 
+      (for-each (lambda (x) (write x out) (newline out)) args)
       (exit 70) ) ) )
 
-(set! syntax-error ##sys#syntax-error-hook)
-
-(define (emit-syntax-trace-info info cntr) 
-  (##core#inline "C_emit_syntax_trace_info" info cntr ##sys#current-thread) )
-
 (define (map-llist proc llist)
   (let loop ([llist llist])
     (cond [(null? llist) '()]
@@ -167,6 +162,9 @@
 	((string? x) (string->symbol x))
 	(else (string->symbol (sprintf "~a" x))) ) )
 
+(define (flonum? x)
+  (and (number? x) (not (exact? x))) )
+
 (define (build-lambda-list vars argc rest)
   (let loop ((vars vars) (n argc))
     (cond ((or (zero? n) (null? vars)) (or rest '()))
@@ -341,9 +339,6 @@
 
 (define decompose-lambda-list ##sys#decompose-lambda-list)
 
-(define (process-lambda-documentation id doc)
-  #f)					; Hook this
-
 
 ;;; Profiling instrumentation:
 
@@ -449,7 +444,7 @@
   (let ((names '((captured . cpt) (assigned . set) (boxed . box) (global . glo) (assigned-locally . stl)
 		 (contractable . con) (standard-binding . stb) (foldable . fld) (simple . sim) (inlinable . inl)
 		 (side-effecting . sef) (collapsable . col) (removable . rem) (constant . con)
-		 (undefined . und) (replacing . rpg) (unused . uud) (extended-binding . xtb) (inline-export . ilx)
+		 (undefined . und) (replacing . rpg) (unused . uud) (extended-binding . xtb)
 		 (customizable . cst) (has-unused-parameters . hup) (boxed-rest . bxr) ) ) )
     (lambda (db)
       (##sys#hash-table-for-each
@@ -464,7 +459,7 @@
 		 (begin
 		   (case (caar es)
 		     ((captured assigned boxed global contractable standard-binding foldable assigned-locally
-		       side-effecting collapsable removable undefined replacing unused simple inlinable inline-export
+		       side-effecting collapsable removable undefined replacing unused simple inlinable
 		       has-unused-parameters extended-binding customizable constant boxed-rest)
 		      (printf "\t~a" (cdr (assq (caar es) names))) )
 		     ((unknown)
@@ -761,22 +756,15 @@
   (unless block-compilation
     (with-output-to-file file
       (lambda ()
-	(let ((exports '()))
-	  (##sys#hash-table-for-each
-	   (lambda (sym plist)
-	     (when (and (assq 'global plist) 
-			(assq 'assigned plist)
-			(or (and export-list (memq sym export-list))
-			    (not (memq sym block-globals)) ) )
-	       (set! exports (cons sym exports)) ) )
-	   db)
-	  (for-each 
-	   (lambda (s)
-	     (write s)
-	     (newline) )
-	   (sort exports
-		 (lambda (s1 s2)
-		   (string<? (##sys#slot s1 1) (##sys#slot s2 1)))) ) ) ) ) ) )
+	(##sys#hash-table-for-each
+	 (lambda (sym plist)
+	   (when (and (assq 'global plist) 
+		      (assq 'assigned plist)
+		      (or (and export-list (memq sym export-list))
+			  (not (memq sym block-globals)) ) )
+	     (write sym) 
+	     (newline) ) )
+	 db) ) ) ) )
 
 (define (dump-undefined-globals db)
   (##sys#hash-table-for-each
@@ -1083,7 +1071,7 @@
     [else
      (match type
        [((or 'instance 'instance-ref) cname sname)
-	`(##tinyclos#make-instance-from-pointer ,body ,sname) ] ;XXX eggified, needs better treatment...
+	`(##tinyclos#make-instance-from-pointer ,body ,sname) ]
        [('nonnull-instance cname sname)
 	`(make ,sname 'this ,body) ]
        [_ body] ) ] ) )
@@ -1236,6 +1224,11 @@ Usage: chicken FILENAME OPTION ...
   Language options:
 
     -feature SYMBOL             register feature identifier
+    -ffi-define SYMBOL          define preprocessor macro for ``easy'' FFI parser
+    -ffi-include-path PATH      set include path for ``easy'' FFI parser
+    -ffi-no-include             don't parse include files when encountered by the FFI parser
+    -ffi                        parse and compile C/C++ code and generate Scheme bindings
+    -ffi-parse                  parse C/C++ code without including it
 
   Syntax related options:
 
@@ -1301,6 +1294,7 @@ Usage: chicken FILENAME OPTION ...
 
     -debug MODES                display debugging output for the given modes
     -compress-literals NUMBER   compile literals above threshold as strings
+    -disable-c-syntax-checks    disable syntax check of C code fragments
     -unsafe-libraries           marks the generated file as being linked
                                 with the unsafe runtime system
     -raw                        do not generate implicit init- and exit code			       
@@ -1406,14 +1400,55 @@ EOF
 
 ;;; "#> ... <#" syntax:
 
-(set! ##sys#user-read-hook
-  (let ([old-hook ##sys#user-read-hook])
-    (lambda (char port)
-      (if (char=? #\> char)	       
-	  (let* ((_ (read-char port))		; swallow #\>
-		 (text (scan-sharp-greater-string port)))
-	    `(declare (foreign-declare ,text)) )
-	  (old-hook char port) ) ) ) )
+(define (enable-sharp-greater-read-syntax)
+  (set! ##sys#user-read-hook
+    (let ([old-hook ##sys#user-read-hook])
+      (lambda (char port)
+	(if (char=? #\> char)	       
+	    (let ([_ (read-char port)]		; swallow #\>
+		  [decl #f]
+		  [parse #f]
+		  [exec #f] )
+	      (case (peek-char port)
+		[(#\!)
+		 (read-char port)
+		 (set! parse #t)
+		 (set! decl #t) ]
+		[(#\?)
+		 (read-char port)
+		 (set! parse #t) ]
+		[(#\:)
+		 (read-char port)
+		 (set! exec #t) ]
+		[(#\()
+		 (let ([head (read port)])
+		   (match head
+		     [_ (for-each
+			 (lambda (t)
+			   (case t
+			     [(declare) (set! decl #t)]
+			     [(parse) (set! parse #t)]
+			     [(execute) (set! exec #t)]
+			     [else (quit "invalid tag in `#>(...) ...<#' form" t)] ) )
+			 head) ] ) ) ]
+		[else (set! decl #t)] )
+	      (let ([text (scan-sharp-greater-string port)])
+		(check-c-syntax text)
+		`(begin
+		   ,@(if decl
+			 `((declare (foreign-declare ,text)))
+			 '() )
+		   ,@(if parse
+			 `((declare (foreign-parse ,text)))
+			 '() )
+		   ,@(if exec
+			 (let ([tmp (gensym 'code_)])
+			   `((declare 
+			       (foreign-declare
+				,(sprintf "static C_word ~A() { ~A; return C_SCHEME_UNDEFINED; }\n" tmp text) ) )
+			     (##core#inline ,(symbol->string tmp)) ) )
+			 '() ) ) ) )
+	    (old-hook char port) ) ) ) ) )
 
 (define (scan-sharp-greater-string port)
   (let ([out (open-output-string)])
@@ -1435,6 +1470,8 @@ EOF
 	       (write-char c out)
 	       (loop) ] ) ) ) ) )
 
+(enable-sharp-greater-read-syntax)
+
 
 ;;; Custom declarations:
 
