@@ -37,12 +37,11 @@
 
 (include "parameters")
 
-(cond-expand [(not chicken-compile-shared) (declare (unit tinyclos))]
-	     [else] )
-
 (declare
+  (unit tinyclos)
   (uses extras)
   (usual-integrations)
+  (run-time-macros)
   (fixnum) 
   (disable-interrupts)
   (hide every1 every2 every2-eq? method-caching-enabled method-cache-tag symbol-vector ##tinyclos#slot-ref
@@ -53,13 +52,23 @@
 	build-transitive-closure applicable? more-specific? method-cache-tag top-sort
 	filter-in) )
 
+(cond-expand
+ ((not unsafe)
+  (declare (emit-exports "tinyclos.exports")) )
+ (else) )
+
 (cond-expand
  [paranoia]
  [else
   (declare
     (no-bound-checks)
+    (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
-     every1 every2 getl filter-in ensure-generic add-global-method
+     every1 every2 getl filter-in ensure-generic add-global-method gensym make make-generic
+     make-method compute-apply-methods compute-methods add-method class-of compute-method-morre-specific?
+     call-next-method ##sys#symbol->string compute-cpl compute-slots compute-getter-and-setter
+     fprintf slot-ref subclass? slot-set! allocate-instance compute-apply-generic class-slots
+     class-direct-supers port? input-port? ##sys#bytevector?
      compute-std-cpl top-sort std-tie-breaker build-transitive-closure build-constraints
      %allocate-instance %allocate-entity get-field set-field! lookup-slot-info
      applicable? more-specific? ##sys#error ##sys#signal-hook
@@ -164,6 +173,46 @@
 (register-feature! 'tinyclos)
 
 
+;;; Macros
+
+(define-macro (define-class name supers slots . meta)
+  (##sys#check-syntax 'define-class name 'symbol)
+  (##sys#check-syntax 'define-class supers '#(_ 0))
+  (##sys#check-syntax 'define-class slots '#(_ 0))
+  (##sys#check-syntax 'define-class meta '#(_ 0 1))
+  `(##core#set! ,name
+     (make ,(if (pair? meta) (##sys#slot meta 0) '<class>)
+       'name ',name
+       'direct-supers (list ,@(if (null? supers) '(<object>) supers))
+       'direct-slots (list ,@(map (lambda (s) `',s) slots)) ) ) )
+
+(define-macro (define-generic name . class)
+  (##sys#check-syntax 'define-generic name 'symbol)
+  `(define ,name (make ,(:optional class '<generic>) 'name ',name)) )
+
+(define-macro (define-method head . body)
+  (##sys#check-syntax 'define-method head '(symbol . _))
+  (##sys#check-syntax 'define-method body '#(_ 1))
+  (let gather ([args (##sys#slot head 1)]
+	       [specs '()]
+	       [vars '()] )
+    (if (or (not (pair? args)) 
+	    (memq (car args) '(#!optional #!key #!rest)) )
+	(let ([name (##sys#slot head 0)])
+	  `(##core#set! ,name
+			(##tinyclos#add-global-method
+			 (##core#global-ref ,name)
+			 ',name
+			 (list ,@(reverse specs))
+			 (##core#named-lambda ,name (call-next-method ,@(reverse vars) ,@args) ,@body) ) ) )
+	(let ([arg (##sys#slot args 0)])
+	  (gather (##sys#slot args 1)
+		  (cons (if (pair? arg) (cadr arg) '<top>) specs)
+		  (cons (if (pair? arg) (car arg) arg) vars) ) ) ) ) )
+
+
+;;; Support code
+
 (define (every1 test lst)
   (let loop ([lst lst])
     (or (null? lst)
@@ -1109,10 +1158,11 @@
   (make-method (list <generic>)
     (lambda (call-next-method generic initargs)
       (call-next-method)
+      (unless (%entity? generic)
+	(##sys#error 'initialize "generic is not an entity") )
       (##tinyclos#slot-set! generic 'methods '())
       (%set-entity-name! generic (quick-getl initargs 'name "(unnamed)"))
-      (%set-entity-proc! generic
-			 (lambda args (##sys#error "has no methods" generic))))))
+      (%set-entity-proc! generic (lambda args (##sys#error "has no methods" generic))))))
 
 (add-method initialize
   (make-method (list <method>)
