@@ -1,17 +1,17 @@
 ;;;; utils.scm - Utilities for scripting and file stuff
 ;
-; Copyright (c) 2000-2007, Felix L. Winkelmann
+; Copyright (c) 2000-2006, Felix L. Winkelmann
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
 ; conditions are met:
 ;
 ;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
-;     disclaimer.
+;     disclaimer. 
 ;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
-;     disclaimer in the documentation and/or other materials provided with the distribution.
+;     disclaimer in the documentation and/or other materials provided with the distribution. 
 ;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
-;     products derived from this software without specific prior written permission.
+;     products derived from this software without specific prior written permission. 
 ;
 ; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
 ; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
@@ -23,7 +23,7 @@
 ; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 ; POSSIBILITY OF SUCH DAMAGE.
 ;
-; Send bugs, suggestions and ideas to:
+; Send bugs, suggestions and ideas to: 
 ;
 ; felix@call-with-current-continuation.org
 ;
@@ -48,7 +48,6 @@
   (declare
     (no-procedure-checks-for-usual-bindings)
     (bound-to-procedure
-      apropos-list apropos
       ##sys#string-append reverse port? read-string with-input-from-file command-line-arguments
       for-each-line ##sys#check-port read-line getenv make-pathname file-exists? call-with-output-file
       decompose-pathname string-search absolute-pathname? string-append ##sys#substring string-match
@@ -75,35 +74,6 @@
 (register-feature! 'utils)
 
 
-;;; Environment utilities
-
-(define ##sys#apropos
-  (lambda (patt env)
-    (when (symbol? patt)
-      (set! patt (symbol->string patt)))
-    (when (string? patt)
-      (set! patt (regexp (regexp-escape patt))))
-    (##sys#environment-symbols env
-      (lambda (sym)
-        (not (not (string-search patt (symbol->string sym)))))) ) )
-
-(let ([%apropos-list
-        (lambda (loc patt args)
-	  (let ([env (if (pair? args) (car args) (interaction-environment))])
-	    (##sys#check-structure env 'environment loc)
-	    (unless (or (string? patt) (symbol? patt) (regexp? patt))
-	      (##sys#signal-hook #:type-error loc "bad argument type - not a string, symbol, or regexp" patt))
-	    (##sys#apropos patt env) ) )])
-  (set! apropos-list
-    (lambda (patt . args)
-      (%apropos-list 'apropos-list patt args)))
-  (set! apropos
-    (lambda (patt . args)
-      (for-each
-        (lambda (sym) (display sym) (newline))
-        (%apropos-list 'apropos patt args)) ) ) )
-
-
 ;;; Like `system', but allows format-string and bombs on nonzero return code:
 
 (define system*
@@ -141,91 +111,80 @@
       (##sys#check-string pn 'absolute-pathname?)
       (pair? (string-match rx pn)) ) ) )
 
-(define (chop-pds str pds)
+(define (chop-pds str)
   (and str
-       (let ((len (##sys#size str))
-	     (pdslen (##sys#size pds)) )
-	 (if (and (fx>= len pdslen) 
-		  (##core#inline "C_substring_compare" str pds (fx- len pdslen) 0 pdslen) )
-	     (##sys#substring str 0 (fx- len pdslen))
+       (let ([len (##sys#size str)])
+	 (if (and (fx> len 0) 
+		  (eq? ##sys#pathname-directory-separator (##core#inline "C_subchar" str (fx- len 1))) )
+	     (##sys#substring str 0 (fx- len 1))
 	     str) ) ) )
 
 (let ([string-append string-append]
       [absolute-pathname? absolute-pathname?]
-      [pds0 (string ##sys#pathname-directory-separator)] )
-  (define (conc-dirs dirs pds)
+      [pds (string ##sys#pathname-directory-separator)] )
+  (define (conc-dirs dirs)
     (##sys#check-list dirs 'make-pathname)
     (let loop ([strs dirs])
-      (if (null? strs)
+      (if (null? strs) 
 	  ""
 	  (let ((s1 (car strs)))
 	    (if (zero? (string-length s1))
 		(loop (cdr strs))
-		(string-append (chop-pds (car strs) pds) pds (loop (cdr strs))) ) ) ) ) )
-  (define (canonicalize dir pds)
+		(string-append (chop-pds (car strs)) pds (loop (cdr strs))) ) ) ) ) )
+  (define (canonicalize dir)
     (cond [(or (not dir) (null? dir)) ""]
-	  [(string? dir) (conc-dirs (list dir) pds)]
-	  [else (conc-dirs dir pds)] ) )
-  (define (_make-pathname loc dir file ext pds)
-    (let ([dirs (canonicalize dir pds)]
-	  (pdslen (##sys#size pds))
-	  (ext (or ext ""))
-	  [file (or file "")] )
-      (##sys#check-string file loc)
-      (##sys#check-string ext loc)
-      (##sys#check-string pds loc)
+	  [(string? dir) (conc-dirs (list dir))]
+	  [else (conc-dirs dir)] ) )
+  (define (_make-pathname dir file . ext)
+    (let ([dirs (canonicalize dir)]
+	  [file (or file "")]
+	  [ext (if (pair? ext) (or (car ext) "") "")] )
+      (##sys#check-string file 'make-pathname)
+      (##sys#check-string ext 'make-pathname)
       (string-append
        dirs
        (if (and dir
-		(and (fx>= (##sys#size file) pdslen)
-		     (##core#inline "C_substring_compare" pds file 0 0 pdslen) ) )
-	   (##sys#substring file pdslen (##sys#size file))
+		(fx> (##sys#size file) 0) 
+		(eq? ##sys#pathname-directory-separator (##core#inline "C_subchar" file 0)) )
+	   (##sys#substring file 1 (##sys#size file))
 	   file)
        (if (and (fx> (##sys#size ext) 0)
 		(not (char=? (##core#inline "C_subchar" ext 0) #\.)) )
 	   "."
 	   "")
        ext) ) )
-  (set! make-pathname
-    (lambda (dir file #!optional ext (pds pds0))
-      (_make-pathname 'make-pathname dir file ext pds)))
+  (set! make-pathname _make-pathname)
   (set! make-absolute-pathname
-    (lambda (dir file #!optional ext (pds pds0))
-      (_make-pathname
-       'make-absolute-pathname
-       (let* ([dirs (canonicalize dir pds)]
+    (lambda (dir file . ext)
+      (apply
+       _make-pathname
+       (let* ([dirs (canonicalize dir)]
 	      [dlen (##sys#size dirs)] )
 	 (if (not (absolute-pathname? dirs))
 	     (##sys#string-append pds dirs)
 	     dirs) )
-       file ext pds) ) ) )
+       file
+       ext) ) ) )
 
 (define decompose-pathname
-  (let* ((pds (string ##sys#pathname-directory-separator))
-	 [set (##sys#string-append "\\/\\" pds)]
+  (let* ([set (##sys#string-append "\\/\\" (string ##sys#pathname-directory-separator))]
 	 [rx1 (string-append "^(.*[" set "])?([^" set "]+)(\\.([^" set ".]+))$")]
-	 [rx2 (string-append "^(.*[" set "])?((\\.)?[^" set "]+)$")]
-	 [string-match string-match]
-	 [strip-pds
-	   (lambda (dir)
-	      (and dir
-		(if (string=? dir pds)
-		    dir
-		    (chop-pds dir pds) ) ) )] )
+	 [rx2 (string-append "^(.*[" set "])?((\\.)?[^" set "]+)$")] 
+	 [string-match string-match] )
     (lambda (pn)
       (##sys#check-string pn 'decompose-pathname)
       (if (eq? (##sys#size pn) 0)
-	  (values #f #f #f)
+	  (values #f #f #f) 
 	  (let ([m (string-search rx1 pn)])
 	    (if m
-		(values (strip-pds (cadr m)) (caddr m) (car (cddddr m)))
+		(values (chop-pds (cadr m)) (caddr m) (car (cddddr m)))
 		(let ([m (string-search rx2 pn)])
 		  (if m
-		      (values (strip-pds (cadr m)) (caddr m) #f)
+		      (values (chop-pds (cadr m)) (caddr m) #f)
 		      (values pn #f #f) ) ) ) ) ) ) ) )
 
 (let ([decompose-pathname decompose-pathname])
-  (set! pathname-directory
+  (set! pathname-directory 
     (lambda (pn)
       (let-values ([(dir file ext) (decompose-pathname pn)])
 	dir) ) )
@@ -233,15 +192,15 @@
     (lambda (pn)
       (let-values ([(dir file ext) (decompose-pathname pn)])
 	file) ) )
-  (set! pathname-extension
+  (set! pathname-extension 
     (lambda (pn)
       (let-values ([(dir file ext) (decompose-pathname pn)])
 	ext) ) )
-  (set! pathname-strip-directory
+  (set! pathname-strip-directory 
     (lambda (pn)
       (let-values ([(dir file ext) (decompose-pathname pn)])
 	(make-pathname #f file ext) ) ) )
-  (set! pathname-strip-extension
+  (set! pathname-strip-extension 
     (lambda (pn)
       (let-values ([(dir file ext) (decompose-pathname pn)])
 	(make-pathname dir file) ) ) )
@@ -260,8 +219,8 @@
 
 (define create-temporary-file
   (let ([getenv getenv]
-	[make-pathname make-pathname]
-	[file-exists? file-exists?]
+	[make-pathname make-pathname] 
+	[file-exists? file-exists?] 
 	[call-with-output-file call-with-output-file] )
     (lambda ext
       (let ([dir (or (getenv "TMPDIR") (getenv "TEMP") (getenv "TMP"))]
@@ -270,7 +229,7 @@
 	(let loop ()
 	  (let* ([n (##sys#fudge 16)]
 		 [pn (make-pathname dir (##sys#string-append "t" (number->string n 16)) ext)] )
-	    (if (file-exists? pn)
+	    (if (file-exists? pn) 
 		(loop)
 		(call-with-output-file pn (lambda (p) pn)) ) ) ) ) ) ) )
 
@@ -284,7 +243,7 @@
 	(##sys#check-port port 'for-each-line)
 	(let loop ()
 	  (let ([ln (read-line port)])
-	    (unless (eof-object? ln)
+	    (unless (eof-object? ln) 
 	      (proc ln)
 	      (loop) ) ) ) ) ) ) )
 
@@ -298,7 +257,7 @@
         (with-input-from-file file (cut for-each-line thunk) ) ) )
   (let ((args (command-line-arguments)))
     (if (null? args)
-        ;; If no arguments, take from stdin,
+        ;; If no arguments, take from stdin, 
         (for-each-line thunk)
         ;; otherwise, hit each file named in argv.
         (for-each (lambda (arg) (file-iterator arg thunk)) args))))
